@@ -1,4 +1,5 @@
 package Remind::PDF;
+# SPDX-License-Identifier: GPL-2.0-only
 use strict;
 use warnings;
 
@@ -133,7 +134,7 @@ sub read_one_month
         $line = $in->getline();
         chomp($line);
         if ($line =~ /^\S+ \S+ \S+ \S+ \S+ \S+ \S+$/) {
-                @{$self->{daynames}} = map { s/_/ /g; $_; } (split(/ /, $line));
+                @{$self->{daynames}} = map { s/_/ /g; $_; } (split(/ /, $line)); ## no critic
         } else {
                 return (undef, "Cannot interpret line: $line");
         }
@@ -212,7 +213,7 @@ hash keys found in the newer "remind -pp" JSON output.
 sub parse_oldstyle_line
 {
         my ($self, $line) = @_;
-        return undef unless $line =~ m|^(\d+)/(\d+)/(\d+) (\S+) (\S+) (\S+) (\S+) (.*)$|;
+        return unless $line =~ m|^(\d+)/(\d+)/(\d+) (\S+) (\S+) (\S+) (\S+) (.*)$|;
 
         my $hash = {
                 date => "$1-$2-$3",
@@ -227,14 +228,16 @@ sub parse_oldstyle_line
 
         if ($hash->{passthru}) {
                 if ($hash->{passthru} =~ /^(shade|color|colour)$/i) {
-                        if ($hash->{body} =~ /^\s*(\d+)\s+(\d+)\s+(\d+)/) {
+                        if ($hash->{body} =~ /^\s*(\d+)\s+(\d+)\s+(\d+)\s*(.*)/) {
                                 $hash->{r} = $1;
                                 $hash->{g} = $2;
                                 $hash->{b} = $3;
-                        } elsif ($hash->{body} =~ /^\s*(\d+)/) {
+                                $hash->{body} = $4;
+                        } elsif ($hash->{body} =~ /^\s*(\d+)\s*(.*)/) {
                                 $hash->{r} = $1;
                                 $hash->{g} = $1;
                                 $hash->{b} = $1;
+                                $hash->{body} = $2;
                         }
                 }
         }
