@@ -5,11 +5,12 @@
 /*  The code for generating a calendar.                        */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2022 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2023 by Dianne Skoll                    */
+/*  SPDX-License-Identifier: GPL-2.0-only                      */
 /*                                                             */
 /***************************************************************/
 
-#define _XOPEN_SOURCE
+#define _XOPEN_SOURCE 600
 #include "config.h"
 #include "custom.h"
 
@@ -147,6 +148,47 @@ static char *VT100Colors[2][2][2][2] /* [Br][R][G][B] */ = {
   }
 };
 
+static char *VT100BGColors[2][2][2] /* [R][G][B] */ = {
+  {
+      {
+          /* 0, 0, 0 = Black   */ "\x1B[0;40m",
+          /* 0, 0, 1 = Blue    */ "\x1B[0;44m"
+      },
+      {
+          /* 0, 1, 0 = Green   */ "\x1B[0;42m",
+          /* 0, 1, 1 = Cyan    */ "\x1B[0;46m"
+      }
+  },
+  {
+      {
+          /* 1, 0, 0 = Red     */ "\x1B[0;41m",
+          /* 1, 0, 1 = Magenta */ "\x1B[0;45m"
+      },
+      {
+          /* 1, 1, 0 = Yellow  */ "\x1B[0;43m",
+          /* 1, 1, 1 = White   */ "\x1B[0;47m"
+      }
+  }
+};
+
+/* Moon phase icons in UTF-8 */
+static char const *moonphase_emojis[] = {
+    "\xF0\x9F\x8C\x91",
+    "\xF0\x9F\x8C\x93",
+    "\xF0\x9F\x8C\x95",
+    "\xF0\x9F\x8C\x97"
+};
+
+/* Moon phases for each day 1-31, up to 32 chars per moon-phase string
+   including termination \0 */
+static char moons[32][32];
+
+/* Week indicators */
+static char weeks[32][32];
+
+/* Background colors of each day 1-31, rgb */
+static int bgcolor[32][3];
+
 static struct line_drawing *linestruct;
 #define DRAW(x) fputs(linestruct->x, stdout)
 
@@ -229,13 +271,14 @@ static struct xterm256_colors XTerm256Colors[] =
 
 /* Global variables */
 static CalEntry *CalColumn[7];
+static int ColToDay[7];
 
 static int ColSpaces;
 
 static int DidAMonth;
 static int DidADay;
 
-static void ColorizeEntry(CalEntry const *e);
+static void ColorizeEntry(CalEntry const *e, int clamp);
 static void SortCol (CalEntry **col);
 static void DoCalendarOneWeek (int nleft);
 static void DoCalendarOneMonth (void);
@@ -244,18 +287,58 @@ static void WriteWeekHeaderLine (void);
 static void WritePostHeaderLine (void);
 static void PrintLeft (char const *s, int width, char pad);
 static void PrintCentered (char const *s, int width, char *pad);
-static int WriteOneCalLine (void);
+static int WriteOneCalLine (int dse, int wd);
 static int WriteOneColLine (int col);
 static void GenerateCalEntries (int col);
 static void WriteCalHeader (void);
 static void WriteCalTrailer (void);
 static int DoCalRem (ParsePtr p, int col);
-static void WriteSimpleEntries (int col, int jul);
+static void WriteSimpleEntries (int col, int dse);
 static void WriteTopCalLine (void);
 static void WriteBottomCalLine (void);
 static void WriteIntermediateCalLine (void);
 static void WriteCalDays (void);
 
+static int
+DayOf(int dse)
+{
+    int d;
+    FromDSE(dse, NULL, NULL, &d);
+    return d;
+}
+
+static void
+Backgroundize(int d)
+{
+    if (d < 1 || d > 31) {
+        return;
+    }
+
+    if (!UseBGVTColors) {
+        return;
+    }
+    if (bgcolor[d][0] < 0) {
+        return;
+    }
+    printf("%s", Colorize(bgcolor[d][0], bgcolor[d][1], bgcolor[d][2], 1, 0));
+}
+
+static void
+UnBackgroundize(int d)
+{
+    if (d < 1 || d > 31) {
+        return;
+    }
+
+    if (!UseBGVTColors) {
+        return;
+    }
+    if (bgcolor[d][0] < 0) {
+        return;
+    }
+    printf("%s", Decolorize());
+}
+
 static void
 send_lrm(void)
 {
@@ -344,14 +427,14 @@ void PrintJSONKeyPairString(char const *name, char const *val)
     printf("\",");
 }
 
-void PrintJSONKeyPairDate(char const *name, int jul)
+void PrintJSONKeyPairDate(char const *name, int dse)
 {
     int y, m, d;
-    if (jul == NO_DATE) {
+    if (dse == NO_DATE) {
 	/* Skip it! */
 	return;
     }
-    FromJulian(jul, &y, &m, &d);
+    FromDSE(dse, &y, &m, &d);
     printf("\"");
     PrintJSONString(name);
     printf("\":\"%04d-%02d-%02d\",", y, m+1, d);
@@ -366,7 +449,7 @@ void PrintJSONKeyPairDateTime(char const *name, int dt)
 	return;
     }
     i = dt / MINUTES_PER_DAY;
-    FromJulian(i, &y, &m, &d);
+    FromDSE(i, &y, &m, &d);
     k = dt % MINUTES_PER_DAY;
     h = k / 60;
     i = k % 60;
@@ -392,7 +475,7 @@ void PrintJSONKeyPairTime(char const *name, int t)
 }
 
 #ifdef REM_USE_WCHAR
-static void PutWideChar(wchar_t const wc)
+void PutWideChar(wchar_t const wc)
 {
     char buf[MB_CUR_MAX+1];
     int len;
@@ -414,7 +497,7 @@ get_month_abbrev(char const *mon)
     return buf;
 #else
     char *s;
-    wchar_t tmp_buf[128];
+    wchar_t tmp_buf[128] = {0};
     wchar_t *ws;
     int i;
     int len;
@@ -510,16 +593,13 @@ ClampColor(int *r, int *g, int *b)
 }
 
 char const *
-Decolorize(int r, int g, int b)
+Decolorize(void)
 {
-    if (!strcmp(Colorize(r, g, b), "")) {
-	return "";
-    }
     return "\x1B[0m";
 }
 
 static char const *
-Colorize256(int r, int g, int b)
+Colorize256(int r, int g, int b, int bg, int clamp)
 {
     static char buf[40];
     int best = -1;
@@ -528,7 +608,9 @@ Colorize256(int r, int g, int b)
     struct xterm256_colors *cur;
     size_t i;
 
-    ClampColor(&r, &g, &b);
+    if (clamp) {
+        ClampColor(&r, &g, &b);
+    }
     for (i=0; i<(sizeof(XTerm256Colors) / sizeof(XTerm256Colors[0])); i++) {
 	cur = &XTerm256Colors[i];
 	dist = ((r - cur->r) * (r - cur->r)) +
@@ -539,32 +621,42 @@ Colorize256(int r, int g, int b)
 	    best = (int) i;
 	}
     }
-    cur = &XTerm256Colors[best];
-    sprintf(buf, "\x1B[38;5;%dm", best);
+    if (bg) {
+        sprintf(buf, "\x1B[48;5;%dm", best);
+    } else {
+        sprintf(buf, "\x1B[38;5;%dm", best);
+    }
     return buf;
 }
 
 static char const *
-ColorizeTrue(int r, int g, int b)
+ColorizeTrue(int r, int g, int b, int bg, int clamp)
 {
     static char buf[40];
-    ClampColor(&r, &g, &b);
-    sprintf(buf, "\x1B[38;2;%d;%d;%dm", r, g, b);
+    if (clamp) {
+        ClampColor(&r, &g, &b);
+    }
+    if (bg) {
+        sprintf(buf, "\x1B[48;2;%d;%d;%dm", r, g, b);
+    } else {
+        sprintf(buf, "\x1B[38;2;%d;%d;%dm", r, g, b);
+    }
     return buf;
 }
 
 char const *
-Colorize(int r, int g, int b)
+Colorize(int r, int g, int b, int bg, int clamp)
 {
     int bright = 0;
 
     if (UseTrueColors) {
-	return ColorizeTrue(r, g, b);
+	return ColorizeTrue(r, g, b, bg, clamp);
     }
 
     if (Use256Colors) {
-	return Colorize256(r, g, b);
+	return Colorize256(r, g, b, bg, clamp);
     }
+
     if (r > 128 || g > 128 || b > 128) {
 	bright = 1;
     }
@@ -575,43 +667,112 @@ Colorize(int r, int g, int b)
     if (b > 64) b = 1;
     else b = 0;
 
-    if (TerminalBackground == TERMINAL_BACKGROUND_DARK) {
+    if (clamp && TerminalBackground == TERMINAL_BACKGROUND_DARK && !bg) {
 	/* Convert black-on-black to grey */
 	if (!r && !g && !b) return VT100Colors[1][0][0][0];
     }
-    if (TerminalBackground == TERMINAL_BACKGROUND_LIGHT) {
+    if (clamp && TerminalBackground == TERMINAL_BACKGROUND_LIGHT && !bg) {
 	/* Convert white-on-white to grey */
 	if (r && g && b) return VT100Colors[1][0][0][0];
     }
-    return VT100Colors[bright][r][g][b];
+    if (bg) {
+        return VT100BGColors[r][g][b];
+    } else {
+        return VT100Colors[bright][r][g][b];
+    }
 }
 
-static void ColorizeEntry(CalEntry const *e)
+static void ColorizeEntry(CalEntry const *e, int clamp)
 {
-    printf("%s", Colorize(e->r, e->g, e->b));
+    printf("%s", Colorize(e->r, e->g, e->b, 0, clamp));
 }
 
-static int
-ComputeCalWidth(int x)
+static void
+InitMoonsAndShades(void)
 {
-    struct winsize w;
-    if (x >= 71) {
-	/* Has been set with -w option */
-	return x;
+    int i;
+    /* Initialize the moon array */
+    if (encoding_is_utf8) {
+        for (i=0; i<=31; i++) {
+            moons[i][0] = 0;
+        }
     }
-    if (!isatty(STDOUT_FILENO)) {
-	/* Output is not a TTY... assume 80 */
-	return 80;
+
+    /* Clear SHADEs */
+    if (UseBGVTColors) {
+        for (i=0; i<=31; i++) {
+            bgcolor[i][0] = -1;
+            bgcolor[i][1] = -1;
+            bgcolor[i][2] = -1;
+        }
     }
-    if (ioctl(STDOUT_FILENO, TIOCGWINSZ, &w) < 0) {
-	return 80;
+
+    /* Clear weeks */
+    for(i=0; i<=31; i++) {
+        weeks[i][0] = 0;
     }
-    if (w.ws_col < 71) {
-	return 71;
-    }
-    return w.ws_col;
 }
 
+static void
+SetShadeEntry(int dse, char const *shade)
+{
+    int y, m, d;
+    int r, g, b;
+    /* Don't bother if we're not doing SHADE specials */
+    if (!UseBGVTColors) {
+        return;
+    }
+
+    if (sscanf(shade, "%d %d %d", &r, &g, &b) != 3) {
+        if (sscanf(shade, "%d", &r) != 1) {
+            return;
+        }
+        g = r;
+        b = r;
+    }
+    if (r < 0 || g < 0 || b < 0 || r > 255 || g > 255 || b > 255) {
+        return;
+    }
+    FromDSE(dse, &y, &m, &d);
+    bgcolor[d][0] = r;
+    bgcolor[d][1] = g;
+    bgcolor[d][2] = b;
+}
+
+static void
+SetMoonEntry(int dse, char const *moon)
+{
+    int phase;
+    int y, m, d;
+    char msg[28];
+
+    /* Don't bother unless it's utf-8 */
+    if (!encoding_is_utf8) {
+        return;
+    }
+
+    msg[0] = 0;
+    if (sscanf(moon, "%d %*d %*d %27[^\x01]", &phase, msg) < 4) {
+        if (sscanf(moon, "%d", &phase) != 1) {
+            /* Malformed MOON special; ignore */
+            fprintf(stderr, "Oops 1\n");
+            return;
+        }
+    }
+    if (phase < 0 || phase > 3) {
+        /* Bad phase */
+        fprintf(stderr, "Oops 2\n");
+        return;
+    }
+    FromDSE(dse, &y, &m, &d);
+    if (msg[0]) {
+        snprintf(moons[d], sizeof(moons[d]), "%s %s", moonphase_emojis[phase], msg);
+    } else {
+        snprintf(moons[d], sizeof(moons[d]), "%s", moonphase_emojis[phase]);
+    }
+}
+
+
 /***************************************************************/
 /*                                                             */
 /*  ProduceCalendar                                            */
@@ -640,15 +801,13 @@ void ProduceCalendar(void)
     }
     ShouldCache = 1;
 
-    CalWidth = ComputeCalWidth(CalWidth);
-
     ColSpaces = (CalWidth - 9) / 7;
     CalWidth = 7*ColSpaces + 8;
 
     /* Run the file once to get potentially-overridden day names */
     if (CalMonths) {
-        FromJulian(JulianToday, &y, &m, &d);
-        JulianToday = Julian(y, m, 1);
+        FromDSE(DSEToday, &y, &m, &d);
+        DSEToday = DSE(y, m, 1);
         GenerateCalEntries(-1);
 	DidAMonth = 0;
 	if (PsCal == PSCAL_LEVEL3) {
@@ -663,8 +822,8 @@ void ProduceCalendar(void)
 	}
 	return;
     } else {
-	if (MondayFirst) JulianToday -= (JulianToday%7);
-	else             JulianToday -= ((JulianToday+1)%7);
+	if (MondayFirst) DSEToday -= (DSEToday%7);
+	else             DSEToday -= ((DSEToday+1)%7);
 
         GenerateCalEntries(-1);
 
@@ -692,22 +851,26 @@ static void DoCalendarOneWeek(int nleft)
     int y, m, d, done, i, l, wd;
     char buf[128];
     int LinesWritten = 0;
-    int OrigJul = JulianToday;
+    int OrigDse = DSEToday;
 
+    InitMoonsAndShades();
 /* Fill in the column entries */
     for (i=0; i<7; i++) {
+        ColToDay[i] = DayOf(DSEToday);
 	GenerateCalEntries(i);
-	JulianToday++;
+	DSEToday++;
     }
 
-/* Output the entries */
+    /* Figure out weekday of first column */
 
+    if (MondayFirst) wd = DSEToday % 7;
+    else             wd = (DSEToday + 1) % 7;
+
+/* Output the entries */
 /* If it's "Simple Calendar" format, do it simply... */
     if (DoSimpleCalendar) {
-	if (MondayFirst) wd = JulianToday % 7;
-	else             wd = (JulianToday + 1) % 7;
 	for (i=0; i<7; i++) {
-	    WriteSimpleEntries(i, OrigJul+i-wd);
+	    WriteSimpleEntries(i, OrigDse+i-wd);
 	}
 	return;
     }
@@ -717,13 +880,33 @@ static void DoCalendarOneWeek(int nleft)
     DRAW(tb);
     goff();
     for (i=0; i<7; i++) {
-	FromJulian(OrigJul+i, &y, &m, &d);
+        FromDSE(OrigDse+i, &y, &m, &d);
         char const *mon = get_month_name(m);
-        snprintf(buf, sizeof(buf), "%d %s ", d, get_month_abbrev(mon));
-	if (OrigJul+i == RealToday)
-	    PrintLeft(buf, ColSpaces, '*');
-	else
+        if (moons[d][0]) {
+            if (weeks[d][0]) {
+                snprintf(buf, sizeof(buf), "%d %s %s %s ", d, get_month_abbrev(mon), weeks[d], moons[d]);
+            } else {
+                snprintf(buf, sizeof(buf), "%d %s %s ", d, get_month_abbrev(mon), moons[d]);
+            }
+        } else {
+            if (weeks[d][0]) {
+                snprintf(buf, sizeof(buf), "%d %s %s ", d, get_month_abbrev(mon), weeks[d]);
+            } else {
+                snprintf(buf, sizeof(buf), "%d %s ", d, get_month_abbrev(mon));
+            }
+        }
+	if (OrigDse+i == RealToday) {
+            if (UseVTColors) {
+                printf("\x1B[1m"); /* Bold */
+            }
+	    PrintLeft(buf, ColSpaces-1, '*');
+            if (UseVTColors) {
+                printf("\x1B[0m"); /* Normal */
+            }
+            putchar(' ');
+        } else {
 	    PrintLeft(buf, ColSpaces, ' ');
+        }
 	gon();
 	DRAW(tb);
 	goff();
@@ -734,7 +917,10 @@ static void DoCalendarOneWeek(int nleft)
 	DRAW(tb);
 	goff();
 	for (i=0; i<7; i++) {
+            d = ColToDay[i];
+            Backgroundize(d);
 	    PrintLeft("", ColSpaces, ' ');
+            UnBackgroundize(d);
 	    gon();
 	    DRAW(tb);
 	    goff();
@@ -745,7 +931,7 @@ static void DoCalendarOneWeek(int nleft)
 /* Write the body lines */
     done = 0;
     while (!done) {
-	done = WriteOneCalLine();
+	done = WriteOneCalLine(OrigDse, wd);
 	LinesWritten++;
     }
 
@@ -755,7 +941,10 @@ static void DoCalendarOneWeek(int nleft)
         DRAW(tb);
         goff();
 	for (i=0; i<7; i++) {
+            d = ColToDay[i];
+            Backgroundize(d);
 	    PrintLeft("", ColSpaces, ' ');
+            UnBackgroundize(d);
 	    gon();
 	    DRAW(tb);
 	    goff();
@@ -782,12 +971,14 @@ static void DoCalendarOneMonth(void)
 {
     int y, m, d, mm, yy, i, j;
 
+    InitMoonsAndShades();
+
     if (!DoSimpleCalendar) WriteCalHeader();
 
     DidADay = 0;
 
     if (PsCal) {
-	FromJulian(JulianToday, &y, &m, &d);
+	FromDSE(DSEToday, &y, &m, &d);
 	if (PsCal == PSCAL_LEVEL1) {
 	    printf("%s\n", PSBEGIN);
 	} else if (PsCal == PSCAL_LEVEL2) {
@@ -800,7 +991,7 @@ static void DoCalendarOneMonth(void)
 	}
 	if (PsCal < PSCAL_LEVEL3) {
 	    printf("%s %d %d %d %d\n",
-		   despace(get_month_name(m)), y, DaysInMonth(m, y), (JulianToday+1) % 7,
+		   despace(get_month_name(m)), y, DaysInMonth(m, y), (DSEToday+1) % 7,
 		   MondayFirst);
             for (i=0; i<7; i++) {
                 j=(i+6)%7;
@@ -815,7 +1006,7 @@ static void DoCalendarOneMonth(void)
 	    PrintJSONKeyPairString("monthname", get_month_name(m));
 	    PrintJSONKeyPairInt("year", y);
 	    PrintJSONKeyPairInt("daysinmonth", DaysInMonth(m, y));
-	    PrintJSONKeyPairInt("firstwkday", (JulianToday+1) % 7);
+	    PrintJSONKeyPairInt("firstwkday", (DSEToday+1) % 7);
 	    PrintJSONKeyPairInt("mondayfirst", MondayFirst);
 	    printf("\"daynames\":[\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\"],",
 		   get_day_name(6), get_day_name(0), get_day_name(1), get_day_name(2),
@@ -873,20 +1064,25 @@ static int WriteCalendarRow(void)
     int y, m, d, wd, i, l;
     int done;
     char buf[81];
-    int OrigJul = JulianToday;
+    int OrigDse = DSEToday;
     int LinesWritten = 0;
     int moreleft;
 
 /* Get the date of the first day */
-    FromJulian(JulianToday, &y, &m, &d);
-    if (!MondayFirst) wd = (JulianToday + 1) % 7;
-    else		     wd = JulianToday % 7;
+    FromDSE(DSEToday, &y, &m, &d);
+    if (!MondayFirst) wd = (DSEToday + 1) % 7;
+    else		     wd = DSEToday % 7;
+
+    for (i=0; i<7; i++) {
+        ColToDay[i] = 0;
+    }
 
 /* Fill in the column entries */
     for (i=wd; i<7; i++) {
 	if (d+i-wd > DaysInMonth(m, y)) break;
 	GenerateCalEntries(i);
-	JulianToday++;
+        ColToDay[i] = DayOf(DSEToday);
+	DSEToday++;
     }
 
 /* Output the entries */
@@ -894,7 +1090,7 @@ static int WriteCalendarRow(void)
 /* If it's "Simple Calendar" format, do it simply... */
     if (DoSimpleCalendar) {
 	for (i=wd; i<7 && d+i-wd<=DaysInMonth(m, y); i++) {
-	    WriteSimpleEntries(i, OrigJul+i-wd);
+	    WriteSimpleEntries(i, OrigDse+i-wd);
 	}
 	return (d+7-wd <= DaysInMonth(m, y));
     }
@@ -908,9 +1104,27 @@ static int WriteCalendarRow(void)
 	if (i < wd || d+i-wd>DaysInMonth(m, y))
 	    PrintLeft("", ColSpaces, ' ');
 	else {
-	    sprintf(buf, "%d ", d+i-wd);
-	    if (Julian(y, m, d+i-wd) == RealToday) {
+            if (moons[d+i-wd][0]) {
+                if (weeks[d+i-wd][0]) {
+                    snprintf(buf, sizeof(buf), "%d %s %s ", d+i-wd, weeks[d+i-wd], moons[d+i-wd]);
+                } else {
+                    snprintf(buf, sizeof(buf), "%d %s ", d+i-wd, moons[d+i-wd]);
+                }
+            } else {
+                if (weeks[d+i-wd][0]) {
+                    snprintf(buf, sizeof(buf), "%d %s ", d+i-wd, weeks[d+i-wd]);
+                } else {
+                    snprintf(buf, sizeof(buf), "%d ", d+i-wd);
+                }
+            }
+	    if (DSE(y, m, d+i-wd) == RealToday) {
+                if (UseVTColors) {
+                    printf("\x1B[1m"); /* Bold */
+                }
 		PrintLeft(buf, ColSpaces-1, '*');
+                if (UseVTColors) {
+                    printf("\x1B[0m"); /* Normal */
+                }
 		putchar(' ');
 	    } else {
 		PrintLeft(buf, ColSpaces, ' ');
@@ -926,7 +1140,9 @@ static int WriteCalendarRow(void)
 	DRAW(tb);
 	goff();
 	for (i=0; i<7; i++) {
+            Backgroundize(ColToDay[i]);
 	    PrintLeft("", ColSpaces, ' ');
+            UnBackgroundize(ColToDay[i]);
 	    gon();
 	    DRAW(tb);
 	    goff();
@@ -937,7 +1153,7 @@ static int WriteCalendarRow(void)
 /* Write the body lines */
     done = 0;
     while (!done) {
-	done = WriteOneCalLine();
+	done = WriteOneCalLine(OrigDse, wd);
 	LinesWritten++;
     }
 
@@ -947,7 +1163,9 @@ static int WriteCalendarRow(void)
 	DRAW(tb);
 	goff();
 	for (i=0; i<7; i++) {
+            Backgroundize(ColToDay[i]);
 	    PrintLeft("", ColSpaces, ' ');
+            UnBackgroundize(ColToDay[i]);
 	    gon();
 	    DRAW(tb);
 	    goff();
@@ -1118,19 +1336,25 @@ static void PrintCentered(char const *s, int width, char *pad)
 /*  Write a single line.                                       */
 /*                                                             */
 /***************************************************************/
-static int WriteOneCalLine(void)
+static int WriteOneCalLine(int start_dse, int wd)
 {
     int done = 1, i;
+    int y, m, d;
 
     gon();
     DRAW(tb);
     goff();
     for (i=0; i<7; i++) {
+        FromDSE(start_dse+i, &y, &m, &d);
+        d -= wd;
 	if (CalColumn[i]) {
+            Backgroundize(ColToDay[i]);
 	    if (WriteOneColLine(i)) done = 0;
 	} else {
+            Backgroundize(ColToDay[i]);
 	    PrintCentered("", ColSpaces, " ");
 	}
+        UnBackgroundize(ColToDay[i]);
 	gon();
 	DRAW(tb);
 	goff();
@@ -1159,9 +1383,12 @@ static int WriteOneColLine(int col)
     wchar_t const *wspace;
     int width;
 #endif
-
+    int clamp = 1;
     int numwritten = 0;
-
+    int d = ColToDay[col];
+    if (d && UseBGVTColors && bgcolor[d][0] != -1) {
+        clamp = 0;
+    }
     /* Print as many characters as possible within the column */
 #ifdef REM_USE_WCHAR
     if (e->wc_text) {
@@ -1204,7 +1431,7 @@ static int WriteOneColLine(int col)
 
 	/* Colorize reminder if necessary */
 	if (UseVTColors && e->is_color) {
-	    ColorizeEntry(e);
+	    ColorizeEntry(e, clamp);
 	}
 
 	/* If we couldn't find a space char, print what we have. */
@@ -1241,9 +1468,10 @@ static int WriteOneColLine(int col)
 	    }
 	}
 
-	/* Decolorize reminder if necessary */
+	/* Decolorize reminder if necessary, but keep any SHADE */
 	if (UseVTColors && e->is_color) {
-	    printf("%s", Decolorize(e->r, e->g, e->b));
+	    printf("%s", Decolorize());
+            Backgroundize(d);
 	}
 
         /* Possibly send lrm control sequence */
@@ -1296,7 +1524,7 @@ static int WriteOneColLine(int col)
 
 	/* Colorize reminder if necessary */
 	if (UseVTColors && e->is_color) {
-	    ColorizeEntry(e);
+	    ColorizeEntry(e, clamp);
 	}
 
 	/* If we couldn't find a space char, print what we have. */
@@ -1324,9 +1552,10 @@ static int WriteOneColLine(int col)
 	    }
 	}
 
-	/* Decolorize reminder if necessary */
+	/* Decolorize reminder if necessary, but keep SHADE */
 	if (UseVTColors && e->is_color) {
-	    printf("%s", Decolorize(e->r, e->g, e->b));
+	    printf("%s", Decolorize());
+            Backgroundize(d);
 	}
 
 	/* Flesh out the rest of the column */
@@ -1420,6 +1649,7 @@ static void GenerateCalEntries(int col)
 	    case T_Exit:    DoExit(&p);	     break;
 	    case T_Set:     r=DoSet(&p);     break;
 	    case T_Fset:    r=DoFset(&p);    break;
+	    case T_Funset:  r=DoFunset(&p);  break;
 	    case T_UnSet:   r=DoUnset(&p);   break;
 	    case T_Clr:     r=DoClear(&p);   break;
 	    case T_Flush:   r=DoFlush(&p);   break;
@@ -1472,7 +1702,7 @@ static void WriteCalHeader(void)
     char buf[80];
     int y, m, d;
 
-    FromJulian(JulianToday, &y, &m, &d);
+    FromDSE(DSEToday, &y, &m, &d);
     sprintf(buf, "%s %d", get_month_name(m), y);
 
     WriteTopCalLine();
@@ -1515,7 +1745,7 @@ static int DoCalRem(ParsePtr p, int col)
     TimeTrig tim;
     Value v;
     int r, err;
-    int jul;
+    int dse;
     CalEntry *CurCol;
     CalEntry *e;
     char const *s, *s2;
@@ -1600,14 +1830,27 @@ static int DoCalRem(ParsePtr p, int col)
 	    DBufFree(&buf);
 	}
 	trig.typ = tok.val;
-	jul = LastTriggerDate;
+        if (trig.typ == MSG_TYPE ||
+            trig.typ == CAL_TYPE ||
+            trig.typ == MSF_TYPE) {
+            is_color = (
+                DefaultColorR != -1
+                && DefaultColorG != -1
+                && DefaultColorB != -1);
+            if (is_color) {
+                col_r = DefaultColorR;
+                col_g = DefaultColorG;
+                col_b = DefaultColorB;
+            }
+        }
+	dse = LastTriggerDate;
 	if (!LastTrigValid) {
 	    FreeTrig(&trig);
 	    return OK;
 	}
     } else {
 	/* Calculate the trigger date */
-	jul = ComputeTrigger(trig.scanfrom, &trig, &tim, &r, 1);
+	dse = ComputeTrigger(trig.scanfrom, &trig, &tim, &r, 1);
 	if (r) {
             if (r == E_CANT_TRIG && trig.maybe_uncomputable) {
                 r = OK;
@@ -1619,7 +1862,7 @@ static int DoCalRem(ParsePtr p, int col)
 
     /* Add to global OMITs if so indicated */
     if (trig.addomit) {
-        r = AddGlobalOmit(jul);
+        r = AddGlobalOmit(dse);
         if (r) {
 	    FreeTrig(&trig);
             return r;
@@ -1663,10 +1906,49 @@ static int DoCalRem(ParsePtr p, int col)
 	}
     }
     if (trig.typ == PASSTHRU_TYPE) {
-	if (!PsCal && StrCmpi(trig.passthru, "COLOR") && StrCmpi(trig.passthru, "COLOUR")) {
+        if (!PsCal && !StrCmpi(trig.passthru, "SHADE")) {
+            if (dse == DSEToday) {
+                DBufInit(&obuf);
+                r = DoSubst(p, &obuf, &trig, &tim, dse, CAL_MODE);
+                if (r) {
+                    DBufFree(&obuf);
+                    FreeTrig(&trig);
+                    return r;
+                }
+                SetShadeEntry(dse, DBufValue(&obuf));
+                DBufFree(&obuf);
+            }
+        }
+        if (!PsCal && !StrCmpi(trig.passthru, "WEEK")) {
+            if (dse == DSEToday) {
+                DBufInit(&obuf);
+                r = DoSubst(p, &obuf, &trig, &tim, dse, CAL_MODE);
+                if (r) {
+                    DBufFree(&obuf);
+                    FreeTrig(&trig);
+                    return r;
+                }
+                sscanf(DBufValue(&obuf), "%31[^\x01]", weeks[DayOf(dse)]);
+                DBufFree(&obuf);
+            }
+        }
+	if (!PsCal && StrCmpi(trig.passthru, "COLOR") && StrCmpi(trig.passthru, "COLOUR") && StrCmpi(trig.passthru, "MOON")) {
 	    FreeTrig(&trig);
 	    return OK;
 	}
+        if (!PsCal && !StrCmpi(trig.passthru, "MOON")) {
+            if (dse == DSEToday) {
+                DBufInit(&obuf);
+                r = DoSubst(p, &obuf, &trig, &tim, dse, CAL_MODE);
+                if (r) {
+                    DBufFree(&obuf);
+                    FreeTrig(&trig);
+                    return r;
+                }
+                SetMoonEntry(dse, DBufValue(&obuf));
+                DBufFree(&obuf);
+            }
+        }
 	if (!StrCmpi(trig.passthru, "COLOR") ||
 	    !StrCmpi(trig.passthru, "COLOUR")) {
 	    is_color = 1;
@@ -1712,9 +1994,9 @@ static int DoCalRem(ParsePtr p, int col)
 
     /* If trigger date == today, add it to the current entry */
     DBufInit(&obuf);
-    if ((jul == JulianToday) ||
+    if ((dse == DSEToday) ||
 	(DoSimpleCalDelta &&
-	 ShouldTriggerReminder(&trig, &tim, jul, &err))) {
+	 ShouldTriggerReminder(&trig, &tim, dse, &err))) {
 	NumTriggered++;
 
 	/* The parse_ptr should not be nested, but just in case... */
@@ -1728,7 +2010,7 @@ static int DoCalRem(ParsePtr p, int col)
 	}
 	if (DoSimpleCalendar || tim.ttime != NO_TIME) {
 	    /* Suppress time if it's not today or if it's a non-COLOR special */
-	    if (jul != JulianToday ||
+	    if (dse != DSEToday ||
 		(trig.typ == PASSTHRU_TYPE &&
 		 StrCmpi(trig.passthru, "COLOUR") &&
 		 StrCmpi(trig.passthru, "COLOR"))) {
@@ -1773,10 +2055,10 @@ static int DoCalRem(ParsePtr p, int col)
 
 	/* In -sa mode, run in ADVANCE mode if we're triggering
 	 * before the actual date */
-	if (jul != JulianToday) {
-	    r = DoSubst(p, &obuf, &trig, &tim, jul, ADVANCE_MODE);
+	if (dse != DSEToday) {
+	    r = DoSubst(p, &obuf, &trig, &tim, dse, ADVANCE_MODE);
 	} else {
-	    r = DoSubst(p, &obuf, &trig, &tim, jul, CAL_MODE);
+	    r = DoSubst(p, &obuf, &trig, &tim, dse, CAL_MODE);
 	}
 	if (r) {
 	    DBufFree(&pre_buf);
@@ -1877,7 +2159,7 @@ static int DoCalRem(ParsePtr p, int col)
 	    e->passthru[0] = 0;
 	}
 	e->pos = e->text;
-	if (jul == JulianToday) {
+	if (dse == DSEToday) {
 	    e->time = tim.ttime;
 	} else {
 	    e->time = NO_TIME;
@@ -2103,13 +2385,13 @@ static void WriteSimpleEntryProtocol2(CalEntry *e, int today)
 /*  Write entries in 'simple calendar' format.                 */
 /*                                                             */
 /***************************************************************/
-static void WriteSimpleEntries(int col, int jul)
+static void WriteSimpleEntries(int col, int dse)
 {
     CalEntry *e = CalColumn[col];
     CalEntry *n;
     int y, m, d;
 
-    FromJulian(jul, &y, &m, &d);
+    FromDSE(dse, &y, &m, &d);
     while(e) {
 	if (DoPrefixLineNo) {
 	    if (PsCal != PSCAL_LEVEL2 && PsCal != PSCAL_LEVEL3) {
@@ -2124,7 +2406,7 @@ static void WriteSimpleEntries(int col, int jul)
 	    }
 	    DidADay = 1;
 	    printf("{\"date\":\"%04d-%02d-%02d\",", y, m+1, d);
-	    WriteSimpleEntryProtocol2(e, jul);
+	    WriteSimpleEntryProtocol2(e, dse);
 	    printf("}");
 	    if (PsCal != PSCAL_LEVEL3) {
 		printf("\n");
