@@ -7,7 +7,8 @@
 /*  commands.                                                  */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2022 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2023 by Dianne Skoll                    */
+/*  SPDX-License-Identifier: GPL-2.0-only                      */
 /*                                                             */
 /***************************************************************/
 
@@ -29,7 +30,7 @@ static int ParseLocalOmit (ParsePtr s, Trigger *t);
 static int ParseScanFrom (ParsePtr s, Trigger *t, int type);
 static int ParsePriority (ParsePtr s, Trigger *t);
 static int ParseUntil (ParsePtr s, Trigger *t, int type);
-static int ShouldTriggerBasedOnWarn (Trigger *t, int jul, int *err);
+static int ShouldTriggerBasedOnWarn (Trigger *t, int dse, int *err);
 static int ComputeTrigDuration(TimeTrig *t);
 
 static int
@@ -55,7 +56,7 @@ int DoRem(ParsePtr p)
     Trigger trig;
     TimeTrig tim;
     int r, err;
-    int jul;
+    int dse;
     DynamicBuffer buf;
     Token tok;
 
@@ -96,9 +97,13 @@ int DoRem(ParsePtr p)
 	FindToken(DBufValue(&buf), &tok);
 	DBufFree(&buf);
 	if (tok.type == T_Empty || tok.type == T_Comment) {
+            r = OK;
+            if (trig.addomit) {
+                r = AddGlobalOmit(LastTriggerDate);
+            }
 	    DBufFree(&buf);
 	    FreeTrig(&trig);
-	    return OK;
+	    return r;
 	}
 	if (tok.type != T_RemType || tok.val == SAT_TYPE) {
 	    DBufFree(&buf);
@@ -120,14 +125,14 @@ int DoRem(ParsePtr p)
 	    DBufFree(&buf);
 	}
 	trig.typ = tok.val;
-	jul = LastTriggerDate;
+	dse = LastTriggerDate;
 	if (!LastTrigValid || PurgeMode) {
 	    FreeTrig(&trig);
 	    return OK;
 	}
     } else {
 	/* Calculate the trigger date */
-	jul = ComputeTrigger(trig.scanfrom, &trig, &tim, &r, 1);
+	dse = ComputeTrigger(trig.scanfrom, &trig, &tim, &r, 1);
 	if (r) {
 	    if (PurgeMode) {
 		PurgeEchoLine("%s: %s\n", "#!P! Problem calculating trigger date", ErrMsg[r]);
@@ -143,14 +148,14 @@ int DoRem(ParsePtr p)
 
     /* Add to global OMITs if so indicated */
     if (trig.addomit) {
-        r = AddGlobalOmit(jul);
+        r = AddGlobalOmit(dse);
         if (r) {
 	    FreeTrig(&trig);
             return r;
         }
     }
     if (PurgeMode) {
-	if (trig.expired || jul < JulianToday) {
+	if (trig.expired || dse < DSEToday) {
 	    if (p->expr_happened) {
 		if (p->nonconst_expr) {
 		    PurgeEchoLine("%s\n", "#!P: Next line may have expired, but contains non-constant expression");
@@ -170,10 +175,10 @@ int DoRem(ParsePtr p)
     }
 
     /* Queue the reminder, if necessary */
-    if (jul == JulianToday &&
+    if (dse == DSEToday &&
 	!(!IgnoreOnce &&
 	  trig.once != NO_ONCE &&
-	  FileAccessDate == JulianToday))
+	  FileAccessDate == DSEToday))
 	QueueReminder(p, &trig, &tim, trig.sched);
     /* If we're in daemon mode, do nothing over here */
     if (Daemon) {
@@ -182,8 +187,8 @@ int DoRem(ParsePtr p)
     }
 
     r = OK;
-    if (ShouldTriggerReminder(&trig, &tim, jul, &err)) {
-	if ( (r=TriggerReminder(p, &trig, &tim, jul)) ) {
+    if (ShouldTriggerReminder(&trig, &tim, dse, &err)) {
+	if ( (r=TriggerReminder(p, &trig, &tim, dse)) ) {
 	    FreeTrig(&trig);
 	    return r;
 	}
@@ -230,6 +235,7 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim, int save_in_globals)
     trig->skip = NO_SKIP;
     trig->once = NO_ONCE;
     trig->addomit = 0;
+    trig->noqueue = 0;
     trig->typ = NO_TYPE;
     trig->scanfrom = NO_DATE;
     trig->from = NO_DATE;
@@ -287,7 +293,7 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim, int save_in_globals)
 	    if (trig->d != NO_DAY) return E_DAY_TWICE;
 	    if (trig->m != NO_MON) return E_MON_TWICE;
 	    if (trig->y != NO_YR)  return E_YR_TWICE;
-	    FromJulian(tok.val, &y, &m, &d);
+	    FromDSE(tok.val, &y, &m, &d);
 	    trig->y = y;
 	    trig->m = m;
 	    trig->d = d;
@@ -298,7 +304,7 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim, int save_in_globals)
 	    if (trig->d != NO_DAY) return E_DAY_TWICE;
 	    if (trig->m != NO_MON) return E_MON_TWICE;
 	    if (trig->y != NO_YR)  return E_YR_TWICE;
-	    FromJulian(tok.val / MINUTES_PER_DAY, &y, &m, &d);
+	    FromDSE(tok.val / MINUTES_PER_DAY, &y, &m, &d);
 	    trig->y = y;
 	    trig->m = m;
 	    trig->d = d;
@@ -431,6 +437,11 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim, int save_in_globals)
             trig->addomit = 1;
             break;
 
+        case T_NoQueue:
+            DBufFree(&buf);
+            trig->noqueue = 1;
+            break;
+
 	case T_Omit:
 	    DBufFree(&buf);
 	    if (trig->omitfunc[0]) {
@@ -470,7 +481,12 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim, int save_in_globals)
 	case T_Tag:
 	    r = ParseToken(s, &buf);
 	    if (r) return r;
+            if (strchr(DBufValue(&buf), ',')) {
+                DBufFree(&buf);
+                return E_PARSE_ERR;
+            }
 	    AppendTag(&(trig->tags), DBufValue(&buf));
+	    DBufFree(&buf);
 	    break;
 
 	case T_Duration:
@@ -543,7 +559,7 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim, int save_in_globals)
     /* Check for some warning conditions */
     if (!s->nonconst_expr) {
         if (trig->y != NO_YR && trig->m != NO_MON && trig->d != NO_DAY && trig->until != NO_UNTIL) {
-            if (Julian(trig->y, trig->m, trig->d) > trig->until) {
+            if (DSE(trig->y, trig->m, trig->d) > trig->until) {
                 Wprint("Warning: UNTIL/THROUGH date earlier than start date");
             }
         }
@@ -564,7 +580,7 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim, int save_in_globals)
 
     /* Set scanfrom to default if not set explicitly */
     if (trig->scanfrom == NO_DATE) {
-        trig->scanfrom = JulianToday;
+        trig->scanfrom = DSEToday;
     }
 
     return OK;
@@ -723,7 +739,7 @@ static int ParseUntil(ParsePtr s, Trigger *t, int type)
 		Eprint("%s: %s", which, ErrMsg[E_DAY_TWICE]);
 		return E_DAY_TWICE;
 	    }
-	    FromJulian(tok.val, &y, &m, &d);
+	    FromDSE(tok.val, &y, &m, &d);
 	    break;
 
 	default:
@@ -736,7 +752,7 @@ static int ParseUntil(ParsePtr s, Trigger *t, int type)
 		DBufFree(&buf);
 		return E_BAD_DATE;
 	    }
-	    t->until = Julian(y, m, d);
+	    t->until = DSE(y, m, d);
 	    PushToken(DBufValue(&buf), s);
 	    DBufFree(&buf);
 	    return OK;
@@ -815,7 +831,7 @@ static int ParseScanFrom(ParsePtr s, Trigger *t, int type)
 		Eprint("%s: %s", word, ErrMsg[E_DAY_TWICE]);
 		return E_DAY_TWICE;
 	    }
-	    FromJulian(tok.val, &y, &m, &d);
+	    FromDSE(tok.val, &y, &m, &d);
 	    break;
 
 	case T_Back:
@@ -839,7 +855,7 @@ static int ParseScanFrom(ParsePtr s, Trigger *t, int type)
 	    if (tok.val < 0) {
 		tok.val = -tok.val;
 	    }
-	    FromJulian(JulianToday - tok.val, &y, &m, &d);
+	    FromDSE(DSEToday - tok.val, &y, &m, &d);
 	    break;
 
 	default:
@@ -852,11 +868,11 @@ static int ParseScanFrom(ParsePtr s, Trigger *t, int type)
 		DBufFree(&buf);
 		return E_BAD_DATE;
 	    }
-	    t->scanfrom = Julian(y, m, d);
+	    t->scanfrom = DSE(y, m, d);
 	    if (type == FROM_TYPE) {
 		t->from = t->scanfrom;
-		if (t->scanfrom < JulianToday) {
-		    t->scanfrom = JulianToday;
+		if (t->scanfrom < DSEToday) {
+		    t->scanfrom = DSEToday;
 		}
 	    } else {
 		t->from = NO_DATE;
@@ -868,6 +884,8 @@ static int ParseScanFrom(ParsePtr s, Trigger *t, int type)
 	}
     }
 }
+
+
 /***************************************************************/
 /*                                                             */
 /*  TriggerReminder                                            */
@@ -875,7 +893,7 @@ static int ParseScanFrom(ParsePtr s, Trigger *t, int type)
 /*  Trigger the reminder if it's a RUN or MSG type.            */
 /*                                                             */
 /***************************************************************/
-int TriggerReminder(ParsePtr p, Trigger *t, TimeTrig *tim, int jul)
+int TriggerReminder(ParsePtr p, Trigger *t, TimeTrig *tim, int dse)
 {
     int r, y, m, d;
     char PrioExpr[VAR_NAME_LEN+25];
@@ -929,25 +947,26 @@ int TriggerReminder(ParsePtr p, Trigger *t, TimeTrig *tim, int jul)
     }
 /* If it's a MSG-type reminder, and no -k option was used, issue the banner. */
     if ((t->typ == MSG_TYPE || t->typ == MSF_TYPE) 
-	&& !NumTriggered && !NextMode && !MsgCommand) {
+	&& !DidMsgReminder && !NextMode && !MsgCommand) {
+        DidMsgReminder = 1;
 	if (!DoSubstFromString(DBufValue(&Banner), &buf,
-			       JulianToday, NO_TIME) &&
+			       DSEToday, NO_TIME) &&
 	    DBufLen(&buf)) {
-	printf("%s\n", DBufValue(&buf));
-	}
+            printf("%s\n", DBufValue(&buf));
+        }
 	DBufFree(&buf);
     }
 
 /* If it's NextMode, process as a ADVANCE_MODE-type entry, and issue
    simple-calendar format. */
     if (NextMode) {
-	if ( (r=DoSubst(p, &buf, t, tim, jul, ADVANCE_MODE)) ) return r;
+	if ( (r=DoSubst(p, &buf, t, tim, dse, ADVANCE_MODE)) ) return r;
 	if (!DBufLen(&buf)) {
 	    DBufFree(&buf);
 	    DBufFree(&pre_buf);
 	    return OK;
 	}
-	FromJulian(jul, &y, &m, &d);
+	FromDSE(dse, &y, &m, &d);
  	sprintf(tmpBuf, "%04d/%02d/%02d ", y, m+1, d);
  	if (DBufPuts(&calRow, tmpBuf) != OK) {
  	    DBufFree(&calRow);
@@ -1030,7 +1049,7 @@ int TriggerReminder(ParsePtr p, Trigger *t, TimeTrig *tim, int jul)
 	    if (!r) {
 		if (!DoCoerce(STR_TYPE, &v)) {
 		    if (is_color) {
-			DBufPuts(&buf, Colorize(red, green, blue));
+			DBufPuts(&buf, Colorize(red, green, blue, 0, 1));
 		    }
 		    if (DBufPuts(&buf, v.v.str) != OK) {
 			DBufFree(&buf);
@@ -1044,9 +1063,9 @@ int TriggerReminder(ParsePtr p, Trigger *t, TimeTrig *tim, int jul)
     }
 
     if (is_color) {
-	DBufPuts(&buf, Colorize(red, green, blue));
+	DBufPuts(&buf, Colorize(red, green, blue, 0, 1));
     }
-    if ( (r=DoSubst(p, &buf, t, tim, jul, NORMAL_MODE)) ) return r;
+    if ( (r=DoSubst(p, &buf, t, tim, dse, NORMAL_MODE)) ) return r;
     if (t->typ != RUN_TYPE) {
 	if (UserFuncExists("msgsuffix") == 1) {
 	    sprintf(PrioExpr, "msgsuffix(%d)", t->priority);
@@ -1055,7 +1074,7 @@ int TriggerReminder(ParsePtr p, Trigger *t, TimeTrig *tim, int jul)
 	    if (!r) {
 		if (!DoCoerce(STR_TYPE, &v)) {
 		    if (is_color) {
-			DBufPuts(&buf, Colorize(red, green, blue));
+			DBufPuts(&buf, Colorize(red, green, blue, 0, 1));
 		    }
 		    if (DBufPuts(&buf, v.v.str) != OK) {
 			DBufFree(&buf);
@@ -1069,7 +1088,7 @@ int TriggerReminder(ParsePtr p, Trigger *t, TimeTrig *tim, int jul)
     }
 
     if (is_color) {
-	DBufPuts(&buf, Decolorize(red, green, blue));
+	DBufPuts(&buf, Decolorize());
     }
 
     if ((!MsgCommand && t->typ == MSG_TYPE) || t->typ == MSF_TYPE) {
@@ -1081,7 +1100,7 @@ int TriggerReminder(ParsePtr p, Trigger *t, TimeTrig *tim, int jul)
 
 /* If we are sorting, just queue it up in the sort buffer */
     if (SortByDate) {
-	if (InsertIntoSortBuffer(jul, tim->ttime, DBufValue(&buf),
+	if (InsertIntoSortBuffer(dse, tim->ttime, DBufValue(&buf),
 				 t->typ, t->priority) == OK) {
 	    DBufFree(&buf);
 	    NumTriggered++;
@@ -1128,20 +1147,20 @@ int TriggerReminder(ParsePtr p, Trigger *t, TimeTrig *tim, int jul)
 /*  triggered.  Sets *err non-zero in event of an error.       */
 /*                                                             */
 /***************************************************************/
-int ShouldTriggerReminder(Trigger *t, TimeTrig *tim, int jul, int *err)
+int ShouldTriggerReminder(Trigger *t, TimeTrig *tim, int dse, int *err)
 {
     int r, omit;
     *err = 0;
 
     /* Handle the ONCE modifier in the reminder. */
-    if (!IgnoreOnce && t->once !=NO_ONCE && FileAccessDate == JulianToday)
+    if (!IgnoreOnce && t->once !=NO_ONCE && FileAccessDate == DSEToday)
 	return 0;
 
-    if (jul < JulianToday) return 0;
+    if (dse < DSEToday) return 0;
 
     /* Don't trigger timed reminders if DontIssueAts is true, and if the
        reminder is for today */
-    if (jul == JulianToday && DontIssueAts && tim->ttime != NO_TIME) {
+    if (dse == DSEToday && DontIssueAts && tim->ttime != NO_TIME) {
 	if (DontIssueAts > 1) {
 	    /* If two or more -a options, then *DO* issue ats that are in the
 	       future */
@@ -1155,7 +1174,7 @@ int ShouldTriggerReminder(Trigger *t, TimeTrig *tim, int jul, int *err)
 
     /* Don't trigger "old" timed reminders */
 /*** REMOVED...
-  if (jul == JulianToday &&
+  if (dse == DSEToday &&
   tim->ttime != NO_TIME &&
   tim->ttime < SystemTime(0) / 60) return 0;
   *** ...UNTIL HERE */
@@ -1166,28 +1185,28 @@ int ShouldTriggerReminder(Trigger *t, TimeTrig *tim, int jul, int *err)
     /* If there's a "warn" function, it overrides any deltas */
     if (t->warn[0] != 0) {
 	if (DeltaOffset) {
-	    if (jul <= JulianToday + DeltaOffset) {
+	    if (dse <= DSEToday + DeltaOffset) {
 		return 1;
 	    }
 	}
-	return ShouldTriggerBasedOnWarn(t, jul, err);
+	return ShouldTriggerBasedOnWarn(t, dse, err);
     }
 
     /* Move back by delta days, if any */
     if (t->delta != NO_DELTA) {
 	if (t->delta < 0)
-	    jul = jul + t->delta;
+	    dse = dse + t->delta;
 	else {
 	    int iter = 0;
 	    int max = MaxSatIter;
 	    r = t->delta;
 	    if (max < r*2) max = r*2;
 	    while(iter++ < max) {
-		if (!r || (jul <= JulianToday)) {
+		if (!r || (dse <= DSEToday)) {
 		    break;
 		}
-		jul--;
-		*err = IsOmitted(jul, t->localomit, t->omitfunc, &omit);
+		dse--;
+		*err = IsOmitted(dse, t->localomit, t->omitfunc, &omit);
 		if (*err) return 0;
 		if (!omit) r--;
 	    }
@@ -1200,7 +1219,7 @@ int ShouldTriggerReminder(Trigger *t, TimeTrig *tim, int jul, int *err)
     }
 
     /* Should we trigger the reminder? */
-    return (jul <= JulianToday + DeltaOffset);
+    return (dse <= DSEToday + DeltaOffset);
 }
 
 /***************************************************************/
@@ -1212,7 +1231,7 @@ int ShouldTriggerReminder(Trigger *t, TimeTrig *tim, int jul, int *err)
 /***************************************************************/
 int DoSatRemind(Trigger *trig, TimeTrig *tt, ParsePtr p)
 {
-    int iter, jul, r, start;
+    int iter, dse, r, start;
     Value v;
     char const *s;
     char const *t;
@@ -1221,25 +1240,25 @@ int DoSatRemind(Trigger *trig, TimeTrig *tt, ParsePtr p)
     iter = 0;
     start = trig->scanfrom;
     while (iter++ < MaxSatIter) {
-	jul = ComputeTriggerNoAdjustDuration(start, trig, tt, &r, 1, 0);
+	dse = ComputeTriggerNoAdjustDuration(start, trig, tt, &r, 1, 0);
 	if (r) {
 	    if (r == E_CANT_TRIG) return OK; else return r;
 	}
-	if (jul != start && trig->duration_days) {
-	    jul = ComputeTriggerNoAdjustDuration(start, trig, tt, &r, 1, trig->duration_days);
+	if (dse != start && trig->duration_days) {
+	    dse = ComputeTriggerNoAdjustDuration(start, trig, tt, &r, 1, trig->duration_days);
 	    if (r) {
 		if (r == E_CANT_TRIG) return OK; else return r;
 	    }
-	} else if (jul == start) {
+	} else if (dse == start) {
 	    if (tt->ttime != NO_TIME) {
 		trig->eventstart = MINUTES_PER_DAY * r + tt->ttime;
 		if (tt->duration != NO_TIME) {
 		    trig->eventduration = tt->duration;
 		}
 	    }
-	    SaveAllTriggerInfo(trig, tt, jul, tt->ttime, 1);
+	    SaveAllTriggerInfo(trig, tt, dse, tt->ttime, 1);
 	}
-	if (jul == -1) {
+	if (dse == -1) {
 	    return E_EXPIRED;
 	}
 	s = p->pos;
@@ -1249,10 +1268,10 @@ int DoSatRemind(Trigger *trig, TimeTrig *tt, ParsePtr p)
 	if (v.type != INT_TYPE && v.type != STR_TYPE) return E_BAD_TYPE;
 	if ((v.type == INT_TYPE && v.v.val) ||
 	    (v.type == STR_TYPE && *v.v.str)) {
-	    AdjustTriggerForDuration(trig->scanfrom, jul, trig, tt, 1);
+	    AdjustTriggerForDuration(trig->scanfrom, dse, trig, tt, 1);
 	    if (DebugFlag & DB_PRTTRIG) {
 		int y, m, d;
-		FromJulian(LastTriggerDate, &y, &m, &d);
+		FromDSE(LastTriggerDate, &y, &m, &d);
 		fprintf(ErrFp, "%s(%d): Trig(satisfied) = %s, %d %s, %d",
 			FileName, LineNo,
 			get_day_name(LastTriggerDate % 7),
@@ -1274,10 +1293,10 @@ int DoSatRemind(Trigger *trig, TimeTrig *tt, ParsePtr p)
 	    return OK;
 	}
 	p->pos = s;
-	if (jul+trig->duration_days < start) {
+	if (dse+trig->duration_days < start) {
 	    start++;
 	} else {
-	    start = jul+trig->duration_days+1;
+	    start = dse+trig->duration_days+1;
 	}
     }
     p->pos = t;
@@ -1384,7 +1403,7 @@ finished:
 /*  function.                                                  */
 /*                                                             */
 /***************************************************************/
-static int ShouldTriggerBasedOnWarn(Trigger *t, int jul, int *err)
+static int ShouldTriggerBasedOnWarn(Trigger *t, int dse, int *err)
 {
     char buffer[VAR_NAME_LEN+32];
     int i;
@@ -1396,7 +1415,7 @@ static int ShouldTriggerBasedOnWarn(Trigger *t, int jul, int *err)
     /* If no proper function exists, barf... */
     if (UserFuncExists(t->warn) != 1) {
 	Eprint("%s: `%s'", ErrMsg[M_BAD_WARN_FUNC], t->warn);
-	return (jul == JulianToday);
+	return (dse == DSEToday);
     }
     for (i=1; ; i++) {
 	sprintf(buffer, "%s(%d)", t->warn, i);
@@ -1405,28 +1424,28 @@ static int ShouldTriggerBasedOnWarn(Trigger *t, int jul, int *err)
 	if (r) {
 	    Eprint("%s: `%s': %s", ErrMsg[M_BAD_WARN_FUNC],
 		   t->warn, ErrMsg[r]);
-	    return (jul == JulianToday);
+	    return (dse == DSEToday);
 	}
 	if (v.type != INT_TYPE) {
 	    DestroyValue(v);
 	    Eprint("%s: `%s': %s", ErrMsg[M_BAD_WARN_FUNC],
 		   t->warn, ErrMsg[E_BAD_TYPE]);
-	    return (jul == JulianToday);
+	    return (dse == DSEToday);
 	}
 
 	/* If absolute value of return is not monotonically
            decreasing, exit */
 	if (i > 1 && abs(v.v.val) >= lastReturnVal) {
-	    return (jul == JulianToday);
+	    return (dse == DSEToday);
 	}
 
 	lastReturnVal = abs(v.v.val);
 	/* Positive values: Just subtract.  Negative values:
            skip omitted days. */
 	if (v.v.val >= 0) {
-	    if (JulianToday + v.v.val == jul) return 1;
+	    if (DSEToday + v.v.val == dse) return 1;
 	} else {
-	    int j = jul;
+	    int j = dse;
 	    int iter = 0;
 	    int max = MaxSatIter;
 	    if (max < v.v.val * 2) max = v.v.val*2;
@@ -1443,7 +1462,7 @@ static int ShouldTriggerBasedOnWarn(Trigger *t, int jul, int *err)
 	        Eprint("Delta: Bad OMITFUNC? %s", ErrMsg[E_CANT_TRIG]);
 	        return 0;
 	    }
-	    if (j == JulianToday) return 1;
+	    if (j == DSEToday) return 1;
 	}
     }
 }
