@@ -5,7 +5,8 @@
 /*  This file contains routines to parse and evaluate          */
 /*  expressions.                                               */
 /*                                                             */
-/*  Copyright 1992-2022 by Dianne Skoll                        */
+/*  Copyright 1992-2023 by Dianne Skoll                        */
+/*  SPDX-License-Identifier: GPL-2.0-only                      */
 /*                                                             */
 /***************************************************************/
 
@@ -253,6 +254,10 @@ static int ParseExprToken(DynamicBuffer *buf, char const **in)
 		    DBufFree(buf);
 		    return E_NO_MEM;
 		}
+                if (!**in) {
+                    DBufFree(buf);
+                    return E_MISS_QUOTE;
+                }
 		continue;
 	    }
 	    c = *(*in)++;
@@ -371,12 +376,10 @@ int Evaluate(char const **s, Var *locals, ParsePtr p)
 	    DBufFree(&ExprBuf);
 	    r = Evaluate(s, locals, p);  /* Leaves the last parsed token in ExprBuf */
 	    if (r) return r;
-	    r = OK;
-	    if (*DBufValue(&ExprBuf) != ')') {
+            if (*DBufValue(&ExprBuf) != ')') {
 		DBufFree(&ExprBuf);
 		return E_MISS_RIGHT_PAREN;
 	    }
-	    if (r) return r;
 	} else if (*DBufValue(&ExprBuf) == '+') {
 	    continue; /* Ignore unary + */
 	}
@@ -658,13 +661,13 @@ int DoCoerce(char type, Value *v)
 	case TIME_TYPE: sprintf(coerce_buf, "%02d%c%02d", v->v.val / 60,
 			       TimeSep, v->v.val % 60);
 	break;
-	case DATE_TYPE: FromJulian(v->v.val, &y, &m, &d);
+	case DATE_TYPE: FromDSE(v->v.val, &y, &m, &d);
 	    sprintf(coerce_buf, "%04d%c%02d%c%02d",
 		    y, DateSep, m+1, DateSep, d);
 	    break;
 	case DATETIME_TYPE:
 	    i = v->v.val / MINUTES_PER_DAY;
-	    FromJulian(i, &y, &m, &d);
+	    FromDSE(i, &y, &m, &d);
 	    k = v->v.val % MINUTES_PER_DAY;
 	    h = k / 60;
 	    i = k % 60;
@@ -1190,9 +1193,9 @@ static int LogNot(void)
 
 /***************************************************************/
 /*                                                             */
-/*  FindFunc                                                   */
+/*  FindOperator                                               */
 /*                                                             */
-/*  Find a function.                                           */
+/*  Find an operator.                                          */
 /*                                                             */
 /***************************************************************/
 Operator *FindOperator(char const *name, Operator where[], int num)
@@ -1201,7 +1204,7 @@ Operator *FindOperator(char const *name, Operator where[], int num)
     int mid, r;
     while (top >= bot) {
 	mid = (top + bot) / 2;
-	r = StrCmpi(name, where[mid].name);
+	r = strcmp(name, where[mid].name);
 	if (!r) return &where[mid];
 	else if (r > 0) bot = mid+1;
 	else top = mid-1;
@@ -1209,6 +1212,20 @@ Operator *FindOperator(char const *name, Operator where[], int num)
     return NULL;
 }
 
+/* Compare two strings case-insensitively, where we KNOW
+   that the second string is definitely lower-case */
+static int strcmp_lcfirst(char const *s1, char const *s2)
+{
+    int r;
+    while (*s1 && *s2) {
+	r = tolower(*s1) - *s2;
+	if (r) return r;
+	s1++;
+	s2++;
+    }
+    return tolower(*s1) - *s2;
+}
+
 /***************************************************************/
 /*                                                             */
 /*  FindFunc                                                   */
@@ -1222,7 +1239,7 @@ BuiltinFunc *FindFunc(char const *name, BuiltinFunc where[], int num)
     int mid, r;
     while (top >= bot) {
 	mid = (top + bot) / 2;
-	r = StrCmpi(name, where[mid].name);
+	r = strcmp_lcfirst(name, where[mid].name);
 	if (!r) return &where[mid];
 	else if (r > 0) bot = mid+1;
 	else top = mid-1;
@@ -1240,12 +1257,31 @@ BuiltinFunc *FindFunc(char const *name, BuiltinFunc where[], int num)
 void PrintValue (Value *v, FILE *fp)
 {
     int y, m, d;
-    char const *s;
+    unsigned char const *s;
 
     if (v->type == STR_TYPE) {
-	s=v->v.str;
+	s = (unsigned char const *) v->v.str;
 	putc('"', fp);
-	for (y=0; y<MAX_PRT_LEN && *s; y++) putc(*s++, fp);
+	for (y=0; y<MAX_PRT_LEN && *s; y++) {
+            switch(*s) {
+            case '\a': fprintf(ErrFp, "\\a"); break;
+            case '\b': fprintf(ErrFp, "\\b"); break;
+            case '\f': fprintf(ErrFp, "\\f"); break;
+            case '\n': fprintf(ErrFp, "\\n"); break;
+            case '\r': fprintf(ErrFp, "\\r"); break;
+            case '\t': fprintf(ErrFp, "\\t"); break;
+            case '\v': fprintf(ErrFp, "\\v"); break;
+            case '"':  fprintf(ErrFp, "\\\""); break;
+            case '\\':  fprintf(ErrFp, "\\\\"); break;
+            default:
+                if (*s < 32) {
+                    fprintf(ErrFp, "\\x%02x", (unsigned int) *s);
+                } else {
+                    putc(*s, ErrFp); break;
+                }
+            }
+            s++;
+        }
 	putc('"',fp);
 	if (*s) fprintf(fp, "...");
     }
@@ -1253,11 +1289,11 @@ void PrintValue (Value *v, FILE *fp)
     else if (v->type == TIME_TYPE) fprintf(fp, "%02d%c%02d", v->v.val / 60,
 					   TimeSep, v->v.val % 60);
     else if (v->type == DATE_TYPE) {
-	FromJulian(v->v.val, &y, &m, &d);
+	FromDSE(v->v.val, &y, &m, &d);
 	fprintf(fp, "%04d%c%02d%c%02d", y, DateSep, m+1, DateSep, d);
     }
     else if (v->type == DATETIME_TYPE) {
-	FromJulian(v->v.val / MINUTES_PER_DAY, &y, &m, &d);
+	FromDSE(v->v.val / MINUTES_PER_DAY, &y, &m, &d);
 	fprintf(fp, "%04d%c%02d%c%02d%c%02d%c%02d", y, DateSep, m+1, DateSep, d, DateTimeSep,
 		(v->v.val % MINUTES_PER_DAY) / 60, TimeSep, (v->v.val % MINUTES_PER_DAY) % 60);
     }
@@ -1330,11 +1366,11 @@ int ParseLiteralTime(char const **s, int *tim)
 /*                                                             */
 /*  ParseLiteralDate                                           */
 /*                                                             */
-/*  Parse a literal date or datetime.  Return result in jul    */
+/*  Parse a literal date or datetime.  Return result in dse    */
 /*  and tim; update s.                                         */
 /*                                                             */
 /***************************************************************/
-int ParseLiteralDate(char const **s, int *jul, int *tim)
+int ParseLiteralDate(char const **s, int *dse, int *tim)
 {
     int y, m, d;
     int r;
@@ -1364,7 +1400,7 @@ int ParseLiteralDate(char const **s, int *jul, int *tim)
     }
     if (!DateOK(y, m, d)) return E_BAD_DATE;
 
-    *jul = Julian(y, m, d);
+    *dse = DSE(y, m, d);
 
     /* Do we have a time part as well? */
     if (**s == ' ' || **s == '@' || **s == 'T' || **s == 't') {
