@@ -8,6 +8,7 @@
 /*                                                             */
 /*  This file is part of REMIND.                               */
 /*  Copyright (C) 1992-2021 by Dianne Skoll                    */
+/*  SPDX-License-Identifier: GPL-2.0-only                      */
 /*                                                             */
 /***************************************************************/
 
@@ -38,9 +39,9 @@ EXTERN  FILE *ErrFp;
 #define IsLeapYear(y) (((y) % 4) ? 0 : ((!((y) % 100) && ((y) % 400)) ? 0 : 1 ))
 #define DaysInMonth(m, y) ((m) != 1 ? MonthDays[m] : 28 + IsLeapYear(y))
 
-#define DestroyValue(x) (void) (((x).type == STR_TYPE && (x).v.str) ? (free((x).v.str),(x).type = ERR_TYPE) : 0)
+#define DestroyValue(x) (void) (((x).type == STR_TYPE && (x).v.str) ? (free((x).v.str),(x).v.str = NULL,(x).type = ERR_TYPE) : 0)
 
-EXTERN	int	JulianToday;
+EXTERN	int	DSEToday;
 EXTERN	int	RealToday;
 EXTERN	int	CurDay;
 EXTERN	int	CurMon;
@@ -58,9 +59,10 @@ EXTERN  INIT(   int     DoSimpleCalendar, 0);
 EXTERN  INIT(   int     DoSimpleCalDelta, 0);
 EXTERN  INIT(   int     DoPrefixLineNo, 0);
 EXTERN	INIT(	int	MondayFirst, 0);
+EXTERN  INIT(   int     AddBlankLines, 1);
 EXTERN	INIT(	int	Iterations, 1);
 EXTERN  INIT(   int     PsCal, 0);
-EXTERN  INIT(   int     CalWidth, -1);
+EXTERN  INIT(   int     CalWidth, 80);
 EXTERN  INIT(   int     CalWeeks, 0);
 EXTERN  INIT(   int     CalMonths, 0);
 EXTERN  INIT(	int 	Hush, 0);
@@ -80,6 +82,7 @@ EXTERN  INIT(   long    SysTime, -1L);
 EXTERN	char	const *InitialFile;
 EXTERN	int	FileAccessDate;
 
+EXTERN  INIT(   int     WeekdayOmits, 0);
 EXTERN  INIT(   int     DontSuppressQuoteMarkers, 0);
 EXTERN  INIT(	int 	DontFork, 0);
 EXTERN  INIT(	int 	DontQueue, 0);
@@ -111,11 +114,13 @@ EXTERN  INIT(   int     LastTriggerTime, 0);
 EXTERN  INIT(   int     ShouldCache, 0);
 EXTERN  char const   *CurLine;
 EXTERN  INIT(   int     NumTriggered, 0);
+EXTERN  INIT(   int     DidMsgReminder, 0);
 EXTERN  int ArgC;
 EXTERN  char const **ArgV;
 EXTERN  INIT(   int     CalLines, CAL_LINES);
 EXTERN  INIT(   int     CalPad, 1);
 EXTERN  INIT(   int     UseVTChars, 0);
+EXTERN  INIT(   int     UseBGVTColors, 0);
 EXTERN  INIT(   int     UseUTF8Chars, 0);
 EXTERN  INIT(   int     UseVTColors, 0);
 EXTERN  INIT(   int     Use256Colors, 0);
