@@ -7,7 +7,8 @@
 /*  in normal mode.                                            */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2022 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2023 by Dianne Skoll                    */
+/*  SPDX-License-Identifier: GPL-2.0-only                      */
 /*                                                             */
 /***************************************************************/
 
@@ -21,6 +22,10 @@
 
 #include <stdlib.h>
 #include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <errno.h>
+
 #include <pwd.h>
 #include <unistd.h>
 #include <sys/ioctl.h>
@@ -43,7 +48,8 @@
  *             simple calendar format.
  *  -r       = Disallow RUN mode
  *  -c[n]    = Produce a calendar for n months (default = 1)
- *  -@[n,m]  = Colorize n=0 VT100 n=1 85 n=2 True m=0 dark terminal m=1 light
+ *  -@[n,m,b]= Colorize n=0 VT100 n=1 85 n=2 True m=0 dark terminal m=1 light
+ *             b=0 ignore SHADE b=1 respect SHADE
  *  -w[n,n,n] = Specify output device width, padding and spacing
  *  -s[n]    = Produce calendar in "simple calendar" format
  *  -p[n]    = Produce calendar in format compatible with rem2ps
@@ -94,6 +100,25 @@ static void AddTrustedUser(char const *username);
 
 static DynamicBuffer default_filename_buf;
 
+static void
+InitCalWidthAndFormWidth(int fd)
+{
+    struct winsize w;
+
+    if (!isatty(fd)) {
+        return;
+    }
+    if (ioctl(fd, TIOCGWINSZ, &w) == 0) {
+        CalWidth = w.ws_col;
+        if (CalWidth < 71) {
+            CalWidth = 71;
+        }
+        FormWidth = w.ws_col - 8;
+        if (FormWidth < 20) FormWidth = 20;
+        if (FormWidth > 500) FormWidth = 500;
+    }
+}
+
 /***************************************************************/
 /*                                                             */
 /*  DefaultFilename                                            */
@@ -140,20 +165,14 @@ void InitRemind(int argc, char const *argv[])
     char const *s;
     int weeks;
     int x;
-    int jul;
+    int dse;
+    int ttyfd;
 
-    jul = NO_DATE;
+    dse = NO_DATE;
 
     /* If stdout is a terminal, initialize $FormWidth to terminal width-8,
        but clamp to [20, 500] */
-    if (isatty(STDOUT_FILENO)) {
-	struct winsize w;
-	if (ioctl(STDOUT_FILENO, TIOCGWINSZ, &w) == 0) {
-	    FormWidth = w.ws_col - 8;
-	    if (FormWidth < 20) FormWidth = 20;
-	    if (FormWidth > 500) FormWidth = 500;
-	}
-    }
+    InitCalWidthAndFormWidth(STDOUT_FILENO);
 
     /* Initialize global dynamic buffers */
     DBufInit(&Banner);
@@ -181,8 +200,8 @@ void InitRemind(int argc, char const *argv[])
 	fprintf(ErrFp, ErrMsg[M_BAD_SYS_DATE], BASE);
 	exit(EXIT_FAILURE);
     }
-    JulianToday = RealToday;
-    FromJulian(JulianToday, &CurYear, &CurMon, &CurDay);
+    DSEToday = RealToday;
+    FromDSE(DSEToday, &CurYear, &CurMon, &CurDay);
 
     /* Initialize Latitude and Longitude */
     set_components_from_lat_and_long();
@@ -233,22 +252,36 @@ void InitRemind(int argc, char const *argv[])
 		    } else if (x == 2) {
 			UseTrueColors = 1;
 		    } else if (x != 0) {
-                        fprintf(ErrFp, "%s: -@n,m: n must be 0, 1 or 2 (assuming 0)\n",
+                        fprintf(ErrFp, "%s: -@n,m,b: n must be 0, 1 or 2 (assuming 0)\n",
                                 argv[0]);
                     }
 		}
 		if (*arg == ',') {
 		    arg++;
-		    PARSENUM(x, arg);
-		    if (x == 0) {
-			TerminalBackground = TERMINAL_BACKGROUND_DARK;
-		    } else if (x == 1) {
-			TerminalBackground = TERMINAL_BACKGROUND_LIGHT;
-		    } else {
-                        fprintf(ErrFp, "%s: -@n,m: m must be 0 or 1\n",
-                                argv[0]);
+                    if (*arg != ',') {
+                        PARSENUM(x, arg);
+                        if (x == 0) {
+                            TerminalBackground = TERMINAL_BACKGROUND_DARK;
+                        } else if (x == 1) {
+                            TerminalBackground = TERMINAL_BACKGROUND_LIGHT;
+                        } else if (x == 2) {
+                            TerminalBackground = TERMINAL_BACKGROUND_UNKNOWN;
+                        } else {
+                            fprintf(ErrFp, "%s: -@n,m,b: m must be 0, 1 or 2 (assuming 2)\n",
+                                    argv[0]);
+                        }
                     }
 		}
+                if (*arg == ',') {
+		    arg++;
+                    PARSENUM(x, arg);
+                    if (x != 0 && x != 1) {
+                        fprintf(ErrFp, "%s: -@n,m,b: b must be 0 or 1 (assuming 0)\n",
+                                argv[0]);
+                        x = 0;
+                    }
+                    UseBGVTColors = x;
+                }
 		break;
 
 	    case 'j':
@@ -495,11 +528,27 @@ void InitRemind(int argc, char const *argv[])
 	    case 'w':
 	    case 'W':
 		if (*arg != ',') {
-		    PARSENUM(CalWidth, arg);
-		    if (CalWidth != 0 && CalWidth < 71) CalWidth = 71;
-		    if (CalWidth == 0) {
-			CalWidth = -1;
-		    }
+                    if (*arg == 't') {
+                        arg++;
+                        /* -wt means get width from /dev/tty */
+                        ttyfd = open("/dev/tty", O_RDONLY);
+                        if (!ttyfd) {
+                            fprintf(stderr, "%s: `-wt': Cannot open /dev/tty: %s\n",
+                                    argv[0], strerror(errno));
+                        } else {
+                            InitCalWidthAndFormWidth(ttyfd);
+                            close(ttyfd);
+                        }
+                    } else {
+                        PARSENUM(CalWidth, arg);
+                        if (CalWidth != 0 && CalWidth < 71) CalWidth = 71;
+                        if (CalWidth == 0) {
+                            CalWidth = -1;
+                        }
+                        FormWidth = CalWidth - 8;
+                        if (FormWidth < 20) FormWidth = 20;
+                        if (FormWidth > 500) FormWidth = 500;
+                    }
 		}
 		if (*arg == ',') {
 		    arg++;
@@ -590,30 +639,30 @@ void InitRemind(int argc, char const *argv[])
 
 	    case T_DateTime:
 		if (SysTime != -1L) Usage();
-		if (m != NO_MON || d != NO_DAY || y != NO_YR || jul != NO_DATE) Usage();
+		if (m != NO_MON || d != NO_DAY || y != NO_YR || dse != NO_DATE) Usage();
 		SysTime = (tok.val % MINUTES_PER_DAY) * 60;
 		DontQueue = 1;
 		Daemon = 0;
-		jul = tok.val / MINUTES_PER_DAY;
+		dse = tok.val / MINUTES_PER_DAY;
 		break;
 
 	    case T_Date:
-		if (m != NO_MON || d != NO_DAY || y != NO_YR || jul != NO_DATE) Usage();
-		jul = tok.val;
+		if (m != NO_MON || d != NO_DAY || y != NO_YR || dse != NO_DATE) Usage();
+		dse = tok.val;
 		break;
 
 	    case T_Month:
-		if (m != NO_MON || jul != NO_DATE) Usage();
+		if (m != NO_MON || dse != NO_DATE) Usage();
 		else m = tok.val;
 		break;
 
 	    case T_Day:
-		if (d != NO_DAY || jul != NO_DATE) Usage();
+		if (d != NO_DAY || dse != NO_DATE) Usage();
 		else d = tok.val;
 		break;
 
 	    case T_Year:
-		if (y != NO_YR || jul != NO_DATE) Usage();
+		if (y != NO_YR || dse != NO_DATE) Usage();
 		else y = tok.val;
 		break;
 
@@ -633,8 +682,8 @@ void InitRemind(int argc, char const *argv[])
 	    Daemon = 0;
 	}
 
-	if (jul != NO_DATE) {
-	    FromJulian(jul, &y, &m, &d);
+	if (dse != NO_DATE) {
+	    FromDSE(dse, &y, &m, &d);
 	}
 /* Must supply date in the form:  day, mon, yr OR mon, yr */
 	if (m != NO_MON || y != NO_YR || d != NO_DAY) {
@@ -652,22 +701,22 @@ void InitRemind(int argc, char const *argv[])
 		fprintf(ErrFp, "%s", BadDate);
 		Usage();
 	    }
-	    JulianToday = Julian(y, m, d);
-	    if (JulianToday == -1) {
+	    DSEToday = DSE(y, m, d);
+	    if (DSEToday == -1) {
 		fprintf(ErrFp, "%s", BadDate);
 		Usage();
 	    }
 	    CurYear = y;
 	    CurMon = m;
 	    CurDay = d;
-	    if (JulianToday != RealToday) IgnoreOnce = 1;
+	    if (DSEToday != RealToday) IgnoreOnce = 1;
 	}
 
     }
 
 /* Figure out the offset from UTC */
     if (CalculateUTC)
-	(void) CalcMinsFromUTC(JulianToday, SystemTime(0)/60,
+	(void) CalcMinsFromUTC(DSEToday, SystemTime(0)/60,
 			       &MinsFromUTC, NULL);
 }
 
@@ -681,7 +730,7 @@ void InitRemind(int argc, char const *argv[])
 #ifndef L_USAGE_OVERRIDE
 void Usage(void)
 {
-    fprintf(ErrFp, "\nREMIND %s (%s version) Copyright 1992-2022 Dianne Skoll\n", VERSION, L_LANGNAME);
+    fprintf(ErrFp, "\nREMIND %s (%s version) Copyright 1992-2023 Dianne Skoll\n", VERSION, L_LANGNAME);
 #ifdef BETA
     fprintf(ErrFp, ">>>> BETA VERSION <<<<\n");
 #endif
@@ -689,7 +738,7 @@ void Usage(void)
     fprintf(ErrFp, "Options:\n");
     fprintf(ErrFp, " -n     Output next occurrence of reminders in simple format\n");
     fprintf(ErrFp, " -r     Disable RUN directives\n");
-    fprintf(ErrFp, " -@[n,m] Colorize COLOR reminders\n");
+    fprintf(ErrFp, " -@[n,m,b] Colorize COLOR/SHADE reminders\n");
     fprintf(ErrFp, " -c[a][n] Produce a calendar for n (default 1) months\n");
     fprintf(ErrFp, " -c[a]+[n] Produce a calendar for n (default 1) weeks\n");
     fprintf(ErrFp, " -w[n[,p[,s]]]  Specify width, padding and spacing of calendar\n");
