@@ -5,7 +5,8 @@
 /*  Header file for language support for various languages.    */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2022 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2023 by Dianne Skoll                    */
+/*  SPDX-License-Identifier: GPL-2.0-only                      */
 /*                                                             */
 /***************************************************************/
 
@@ -23,7 +24,7 @@
 #define ITALIAN    9 /* translated by Valerio Aimale                   */
 #define ROMANIAN  10 /* translated by Liviu Daia                       */
 #define SPANISH   11 /* translated by Rafa Couto                       */
-#define ICELANDIC 12 /* translated by Bjrn Davsson                  */
+#define ICELANDIC 12 /* translated by Björn Davíðsson                  */
 
 /* Add more languages here - but please e-mail dianne@skoll.ca
    to have your favorite language assigned a number.  If you add a
