@@ -8,8 +8,9 @@
 /*                                                             */
 /*  This file is part of REMIND.                               */
 /*                                                             */
-/*  REMIND is Copyright (C) 1992-2022 by Dianne Skoll          */
+/*  REMIND is Copyright (C) 1992-2023 by Dianne Skoll          */
 /*  This file is Copyright (C) 1996-1998 by Liviu Daia         */
+/*  SPDX-License-Identifier: GPL-2.0-only                      */
 /*                                                             */
 /***************************************************************/
 
@@ -84,14 +85,14 @@
 #define L_AMPM_OVERRIDE(ampm, hour)	ampm = (hour < 12) ? (hour<4) ? " noaptea" : " dimineaţa" : (hour > 17) ? " seara" : " după-amiaza";
 #define L_ORDINAL_OVERRIDE		plu = "";
 
-#define L_A_OVER sprintf(s, "%s, %d %s %d", DayName[jul%7], d, MonthName[m], y);
-#define L_C_OVER sprintf(s, "%s", DayName[jul%7]);
-#define L_G_OVER sprintf(s, "%s, %d %s", DayName[jul%7], d, MonthName[m]);
-#define L_J_OVER sprintf(s, "%s, %s %d, %d", DayName[jul%7], MonthName[m], d, y);
-#define L_K_OVER sprintf(s, "%s, %s %d", DayName[jul%7], MonthName[m], d);
+#define L_A_OVER sprintf(s, "%s, %d %s %d", DayName[dse%7], d, MonthName[m], y);
+#define L_C_OVER sprintf(s, "%s", DayName[dse%7]);
+#define L_G_OVER sprintf(s, "%s, %d %s", DayName[dse%7], d, MonthName[m]);
+#define L_J_OVER sprintf(s, "%s, %s %d, %d", DayName[dse%7], MonthName[m], d, y);
+#define L_K_OVER sprintf(s, "%s, %s %d", DayName[dse%7], MonthName[m], d);
 #define L_S_OVER
-#define L_U_OVER sprintf(s, "%s, %d %s %d", DayName[jul%7], d, MonthName[m], y);
-#define L_V_OVER sprintf(s, "%s, %d %s", DayName[jul%7], d, MonthName[m]);
+#define L_U_OVER sprintf(s, "%s, %d %s %d", DayName[dse%7], d, MonthName[m], y);
+#define L_V_OVER sprintf(s, "%s, %d %s", DayName[dse%7], d, MonthName[m]);
 #define L_1_OVER							\
             if (tdiff == 0)						\
                 sprintf(s, L_NOW);					\
