@@ -6,12 +6,15 @@
 /*  routines, etc.                                             */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2022 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2023 by Dianne Skoll                    */
+/*  SPDX-License-Identifier: GPL-2.0-only                      */
 /*                                                             */
 /***************************************************************/
 
+#define _XOPEN_SOURCE 600
 #include "config.h"
 
+#include <errno.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <stdio.h>
@@ -35,6 +38,10 @@
 #endif
 
 #include <sys/types.h>
+#ifdef REM_USE_WCHAR
+#include <wctype.h>
+#include <wchar.h>
+#endif
 
 #include "types.h"
 #include "protos.h"
@@ -67,6 +74,7 @@ int main(int argc, char *argv[])
     ArgV = (char const **) argv;
 
     InitRemind(argc, (char const **) argv);
+    DBufInit(&(LastTrigger.tags));
     ClearLastTriggers();
 
     if (DoCalendar || (DoSimpleCalendar && (!NextMode || PsCal))) {
@@ -124,7 +132,7 @@ int main(int argc, char *argv[])
 	}
 	if (Iterations) {
 	    PerIterationInit();
-	    JulianToday++;
+	    DSEToday++;
 	}
     }
     return 0;
@@ -169,14 +177,16 @@ static void DoReminders(void)
     Parser p;
     int purge_handled;
 
+    DidMsgReminder = 0;
+
     if (!UseStdin) {
 	FileAccessDate = GetAccessDate(InitialFile);
     } else {
-	FileAccessDate = JulianToday;
+	FileAccessDate = DSEToday;
     }
 
     if (FileAccessDate < 0) {
-	fprintf(ErrFp, "%s: `%s'.\n", ErrMsg[E_CANTACCESS], InitialFile);
+	fprintf(ErrFp, "%s: `%s': %s.\n", ErrMsg[E_CANTACCESS], InitialFile, strerror(errno));
 	exit(1);
     }
 
@@ -253,6 +263,7 @@ static void DoReminders(void)
 	    case T_Flush:   r=DoFlush(&p);   break;
 	    case T_Set:     r=DoSet(&p);     break;
 	    case T_Fset:    r=DoFset(&p);    break;
+	    case T_Funset:  r=DoFunset(&p);  break;
 	    case T_UnSet:   r=DoUnset(&p);   break;
 	    case T_Clr:     r=DoClear(&p);   break;
             case T_Debug:   r=DoDebug(&p);   break;
@@ -307,13 +318,16 @@ static void DoReminders(void)
 
 /***************************************************************/
 /*                                                             */
-/*  Julian                                                     */
+/*  DSE                                                        */
 /*                                                             */
-/*  Given day, month, year, return Julian date in days since   */
+/*  DSE stands for "Days Since Epoch"; the Remind epoch is     */
+/*  midnight on 1990-01-01                                     */
+/*                                                             */
+/*  Given day, month, year, return DSE date in days since      */
 /*  1 January 1990.                                            */
 /*                                                             */
 /***************************************************************/
-int Julian(int year, int month, int day)
+int DSE(int year, int month, int day)
 {
     int y1 = BASE-1, y2 = year-1;
 
@@ -327,14 +341,15 @@ int Julian(int year, int month, int day)
 
 /***************************************************************/
 /*                                                             */
-/*  FromJulian                                                 */
+/*  FromDSE                                                    */
 /*                                                             */
-/*  Convert a Julian date to year, month, day.                 */
+/*  Convert a DSE date to year, month, day.  You may supply    */
+/*  NULL for y, m or d if you're not interested in that value  */
 /*                                                             */
 /***************************************************************/
-void FromJulian(int jul, int *y, int *m, int *d)
+void FromDSE(int dse, int *y, int *m, int *d)
 {
-    int try_yr = (jul / 365) + BASE;
+    int try_yr = (dse / 365) + BASE;
     int try_mon = 0;
     int t;
 
@@ -344,26 +359,49 @@ void FromJulian(int jul, int *y, int *m, int *d)
     int y100 = (y2 / 100) - (y1 / 100); /* Don't count multiples of 100... */
     int y400 = (y2 / 400) - (y1 / 400); /* ... but do count multiples of 400 */
 
-    int try_jul= 365 * (try_yr-BASE) + y4 - y100 + y400;
+    int try_dse= 365 * (try_yr-BASE) + y4 - y100 + y400;
 
-    while (try_jul > jul) {
+    while (try_dse > dse) {
 	try_yr--;
-	try_jul -= DaysInYear(try_yr);
+	try_dse -= DaysInYear(try_yr);
     }
-    jul -= try_jul;
+    dse -= try_dse;
 
     t = DaysInMonth(try_mon, try_yr);
-    while (jul >= t) {
-	jul -= t;
+    while (dse >= t) {
+	dse -= t;
 	try_mon++;
 	t = DaysInMonth(try_mon, try_yr);
     }
-    *y = try_yr;
-    *m = try_mon;
-    *d = jul + 1;
+    if (y) {
+        *y = try_yr;
+    }
+    if (m) {
+        *m = try_mon;
+    }
+    if (d) {
+        *d = dse + 1;
+    }
     return;
 }
 
+int JulianToGregorianOffset(int y, int m)
+{
+    int offset = 13;
+    int centuries;
+    int four_centuries;
+    if (y >= 2100) {
+        centuries = (y - 2000) / 100;
+        four_centuries = (y - 2000) / 400;
+        offset += centuries - four_centuries;
+        if (!(y%100) && (y % 400)) {
+            if (m < 2) {
+                offset--;  /* Offset increments in March */
+            }
+        }
+    }
+    return offset;
+}
 /***************************************************************/
 /*                                                             */
 /*  ParseChar                                                  */
@@ -732,7 +770,7 @@ long SystemTime(int realtime)
 /*                                                             */
 /*  SystemDate                                                 */
 /*                                                             */
-/*  Obtains today's date.  Returns Julian date or -1 for       */
+/*  Obtains today's date.  Returns DSE date or -1 for       */
 /*  failure.  (Failure happens if sys date is before BASE      */
 /*  year.)                                                     */
 /*                                                             */
@@ -749,7 +787,7 @@ int SystemDate(int *y, int *m, int *d)
     *m = t->tm_mon;
     *y = t->tm_year + 1900;
 
-    return Julian(*y, *m, *d);
+    return DSE(*y, *m, *d);
 }
 
 
@@ -842,7 +880,7 @@ int DoIfTrig(ParsePtr p)
     unsigned syndrome;
     Trigger trig;
     TimeTrig tim;
-    int jul;
+    int dse;
 
 
     if ((size_t) NumIfs >= IF_NEST) return E_NESTED_IF;
@@ -850,26 +888,26 @@ int DoIfTrig(ParsePtr p)
     else {
 	if ( (r=ParseRem(p, &trig, &tim, 1)) ) return r;
 	if (trig.typ != NO_TYPE) return E_PARSE_ERR;
-	jul = ComputeTrigger(trig.scanfrom, &trig, &tim, &r, 1);
+	dse = ComputeTrigger(trig.scanfrom, &trig, &tim, &r, 1);
 	if (r) {
             if (r != E_CANT_TRIG || !trig.maybe_uncomputable) {
                 if (!Hush || r != E_RUN_DISABLED) {
                     Eprint("%s", ErrMsg[r]);
                 }
             }
-	    syndrome = IF_TRUE | BEFORE_ELSE;
+	    syndrome = IF_FALSE | BEFORE_ELSE;
 	}
 	else {
-	    if (ShouldTriggerReminder(&trig, &tim, jul, &err)) {
+	    if (ShouldTriggerReminder(&trig, &tim, dse, &err)) {
 		syndrome = IF_TRUE | BEFORE_ELSE;
 	    } else {
 		syndrome = IF_FALSE | BEFORE_ELSE;
-		if (PurgeMode) {
-		    PurgeEchoLine("%s\n", "#!P: The next IFTRIG did not trigger.");
-		    PurgeEchoLine("%s\n", "#!P: REM statements in IFTRIG block not checked for purging.");
-		}
 	    }
 	}
+        if (syndrome == (IF_FALSE | BEFORE_ELSE) && PurgeMode) {
+            PurgeEchoLine("%s\n", "#!P: The next IFTRIG did not trigger.");
+            PurgeEchoLine("%s\n", "#!P: REM statements in IFTRIG block not checked for purging.");
+        }
 	FreeTrig(&trig);
     }
     NumIfs++;
@@ -1124,7 +1162,7 @@ int DoErrMsg(ParsePtr p)
     DBufInit(&buf);
     t.typ = MSG_TYPE;
     tt.ttime = SystemTime(0) / 60;
-    if ( (r=DoSubst(p, &buf, &t, &tt, JulianToday, NORMAL_MODE)) ) {
+    if ( (r=DoSubst(p, &buf, &t, &tt, DSEToday, NORMAL_MODE)) ) {
 	return r;
     }
     s = DBufValue(&buf);
@@ -1154,28 +1192,30 @@ static int FoldArray[2][7] = {
     {2024, 2008, 2020, 2004, 2016, 2000, 2012}
 };
 
-int CalcMinsFromUTC(int jul, int tim, int *mins, int *isdst)
+int CalcMinsFromUTC(int dse, int tim, int *mins, int *isdst)
 {
 
-/* Convert jul and tim to an Unix tm struct */
+/* Convert dse and tim to an Unix tm struct */
     int yr, mon, day;
     int tdiff;
     struct tm local, utc, *temp;
     time_t loc_t, utc_t;
     int isdst_tmp;
 
-    FromJulian(jul, &yr, &mon, &day);
+    FromDSE(dse, &yr, &mon, &day);
 
 /* If the year is greater than 2037, some Unix machines have problems.
    Fold it back to a "similar" year and trust that the UTC calculations
    are still valid... */
     if (FoldYear && yr>2037) {
-	jul = Julian(yr, 0, 1);
-	yr = FoldArray[IsLeapYear(yr)][jul%7];
+	dse = DSE(yr, 0, 1);
+	yr = FoldArray[IsLeapYear(yr)][dse%7];
     }
     local.tm_sec = 0;
     local.tm_min = tim % 60;
     local.tm_hour = tim / 60;
+
+
     local.tm_mday = day;
     local.tm_mon = mon;
     local.tm_year = yr-1900;
@@ -1204,6 +1244,144 @@ int CalcMinsFromUTC(int jul, int tim, int *mins, int *isdst)
     return 0;
 }
 
+static char const *OutputEscapeSequences(char const *s, int print)
+{
+    while (*s == 0x1B && *(s+1) == '[') {
+        if (print) putchar(*s);
+        s++;
+        if (print) putchar(*s);
+        s++;
+        while (*s && (*s < 0x40 || *s > 0x7E)) {
+            if (print) putchar(*s);
+            s++;
+        }
+        if (*s) {
+            if (print) putchar(*s);
+            s++;
+        }
+    }
+    return s;
+}
+
+#ifdef REM_USE_WCHAR
+#define ISWBLANK(c) (iswspace(c) && (c) != '\n')
+static wchar_t const *OutputEscapeSequencesWS(wchar_t const *s, int print)
+{
+    while (*s == 0x1B && *(s+1) == '[') {
+        if (print) PutWideChar(*s);
+        s++;
+        if (print) PutWideChar(*s);
+        s++;
+        while (*s && (*s < 0x40 || *s > 0x7E)) {
+            if (print) PutWideChar(*s);
+            s++;
+        }
+        if (*s) {
+            if (print) PutWideChar(*s);
+            s++;
+        }
+    }
+    return s;
+}
+
+
+static void
+FillParagraphWCAux(wchar_t const *s)
+{
+    int line = 0;
+    int i, j;
+    int doublespace = 1;
+    int pendspace;
+    int len;
+    wchar_t const *t;
+
+    int roomleft;
+    /* Start formatting */
+    while(1) {
+
+	/* If it's a carriage return, output it and start new paragraph */
+	if (*s == '\n') {
+	    putchar('\n');
+	    s++;
+	    line = 0;
+	    while(ISWBLANK(*s)) s++;
+	    continue;
+	}
+	if (!*s) {
+	    return;
+	}
+	/* Over here, we're at the beginning of a line.  Emit the correct
+	   number of spaces */
+	j = line ? SubsIndent : FirstIndent;
+	for (i=0; i<j; i++) {
+	    putchar(' ');
+	}
+
+	/* Calculate the amount of room left on this line */
+	roomleft = FormWidth - j;
+	pendspace = 0;
+
+	/* Emit words until the next one won't fit */
+	while(1) {
+	    while(ISWBLANK(*s)) s++;
+	    if (*s == '\n') break;
+            while(1) {
+                t = s;
+                s = OutputEscapeSequencesWS(s, 1);
+                if (s == t) break;
+                while(ISWBLANK(*s)) s++;
+            }
+	    t = s;
+            len = 0;
+	    while(*s && !iswspace(*s)) {
+                if (*s == 0x1B && *(s+1) == '[') {
+                    s = OutputEscapeSequencesWS(s, 0);
+                    continue;
+                }
+                len += wcwidth(*s);
+                s++;
+            }
+	    if (s == t) {
+		return;
+	    }
+	    if (!pendspace || len+pendspace <= roomleft) {
+		for (i=0; i<pendspace; i++) {
+		    putchar(' ');
+		}
+		while(t < s) {
+                    PutWideChar(*t);
+		    if (strchr(EndSent, *t)) doublespace = 2;
+		    else if (!strchr(EndSentIg, *t)) doublespace = 1;
+		    t++;
+		}
+	    } else {
+		s = t;
+		putchar('\n');
+		line++;
+		break;
+	    }
+	    roomleft -= len+doublespace;
+	    pendspace = doublespace;
+	}
+    }
+}
+
+static int
+FillParagraphWC(char const *s)
+{
+    size_t len;
+    wchar_t *buf;
+
+    len = mbstowcs(NULL, s, 0);
+    if (len == (size_t) -1) return E_NO_MEM;
+    buf = calloc(len+1, sizeof(wchar_t));
+    if (!buf) return E_NO_MEM;
+    (void) mbstowcs(buf, s, len+1);
+    FillParagraphWCAux(buf);
+    free(buf);
+    return OK;
+}
+#endif
 /***************************************************************/
 /*                                                             */
 /*  FillParagraph                                              */
@@ -1230,11 +1408,17 @@ void FillParagraph(char const *s)
     char const *t;
 
     int roomleft;
-
     if (!s || !*s) return;
 
     /* Skip leading spaces */
     while(ISBLANK(*s)) s++;
+    if (!*s) return;
+
+#ifdef REM_USE_WCHAR
+    if (FillParagraphWC(s) == OK) {
+        return;
+    }
+#endif
 
     /* Start formatting */
     while(1) {
@@ -1265,10 +1449,23 @@ void FillParagraph(char const *s)
 	while(1) {
 	    while(ISBLANK(*s)) s++;
 	    if (*s == '\n') break;
+            while(1) {
+                t = s;
+                s = OutputEscapeSequences(s, 1);
+                if (s == t) break;
+                while(ISBLANK(*s)) s++;
+            }
 	    t = s;
-	    while(*s && !isspace(*s)) s++;
-	    len = s - t;
-	    if (!len) {
+            len = 0;
+	    while(*s && !isspace(*s)) {
+                if (*s == 0x1B && *(s+1) == '[') {
+                    s = OutputEscapeSequences(s, 0);
+                    continue;
+                }
+                s++;
+                len++;
+            }
+	    if (s == t) {
 		return;
 	    }
 	    if (!pendspace || len+pendspace <= roomleft) {
@@ -1414,6 +1611,7 @@ ClearLastTriggers(void)
     LastTrigger.warn[0] = 0;
     LastTrigger.omitfunc[0] = 0;
     LastTrigger.passthru[0] = 0;
+    DBufFree(&(LastTrigger.tags));
 
     LastTimeTrig.ttime = NO_TIME;
     LastTimeTrig.delta = NO_DELTA;
@@ -1434,8 +1632,10 @@ SaveAllTriggerInfo(Trigger const *t, TimeTrig const *tt, int trigdate, int trigt
 void
 SaveLastTrigger(Trigger const *t)
 {
+    DBufFree(&(LastTrigger.tags));
     memcpy(&LastTrigger, t, sizeof(LastTrigger));
     DBufInit(&(LastTrigger.tags));
+    DBufPuts(&(LastTrigger.tags), DBufValue(&(t->tags)));
 }
 
 void
@@ -1451,7 +1651,7 @@ System(char const *cmd)
     int r;
     r = system(cmd);
     if (r == 0) {
-	r = 1;
+        return;
     }
 }
 
