@@ -5,7 +5,8 @@
 /*  Calculations for figuring out moon phases.                 */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2022 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2023 by Dianne Skoll                    */
+/*  SPDX-License-Identifier: GPL-2.0-only                      */
 /*                                                             */
 /***************************************************************/
 
@@ -143,7 +144,7 @@ static double phase (double, double *, double *, double *, double *, double *, d
 /*                                                             */
 /*  jdate                                                      */
 /*                                                             */
-/*  Convert a date and time to Julian day and fraction.        */
+/*  Convert a date and time to DSE day and fraction.        */
 /*                                                             */
 /***************************************************************/
 static long jdate(int y, int mon, int day)
@@ -400,9 +401,9 @@ static double phase(double pdate,
 		    double *suangdia)
 {
 
-    double Day, N, M, Ec, Lambdasun, ml, MM, MN, Ev, Ae, A3, MmP,
-	mEc, A4, lP, V, lPP, NP, y, x, Lambdamoon,
-	MoonAge, MoonPhase,
+    double Day, N, M, Ec, Lambdasun, ml, MM, Ev, Ae, A3, MmP,
+	mEc, A4, lP, V, lPP,
+	MoonAge, Phase,
 	MoonDist, MoonDFrac, MoonAng,
 	F, SunDist, SunAng;
 
@@ -431,9 +432,6 @@ static double phase(double pdate,
     /* Moon's mean anomaly */
     MM = fixangle(ml - 0.1114041 * Day - mmlongp);
 
-    /* Moon's ascending node mean longitude */
-    MN = fixangle(mlnode - 0.0529539 * Day);
-
     /* Evection */
     Ev = 1.2739 * sin(torad(2 * (ml - Lambdasun) - MM));
 
@@ -461,26 +459,13 @@ static double phase(double pdate,
     /* 1 longitude */
     lPP = lP + V;
 
-    /* Corrected longitude of the node */
-    NP = MN - 0.16 * sin(torad(M));
-
-    /* Y inclination coordinate */
-    y = sin(torad(lPP - NP)) * cos(torad(minc));
-
-    /* X inclination coordinate */
-    x = cos(torad(lPP - NP));
-
-    /* Ecliptic longitude */
-    Lambdamoon = todeg(atan2(y, x));
-    Lambdamoon += NP;
-
     /* Calculation of the phase of the Moon */
 
     /* Age of the Moon in degrees */
     MoonAge = lPP - Lambdasun;
 
     /* Phase of the Moon */
-    MoonPhase = (1 - cos(torad(MoonAge))) / 2;
+    Phase = (1 - cos(torad(MoonAge))) / 2;
 
     /* Calculate distance of moon from the centre of the Earth */
 
@@ -492,7 +477,7 @@ static double phase(double pdate,
     MoonDFrac = MoonDist / msmax;
     MoonAng = mangsiz / MoonDFrac;
 
-    if(pphase)   *pphase = MoonPhase;
+    if(pphase)   *pphase = Phase;
     if(mage)     *mage = synmonth * (fixangle(MoonAge) / 360.0);
     if(dist)     *dist = MoonDist;
     if(angdia)   *angdia = MoonAng;
@@ -520,10 +505,10 @@ int MoonPhase(int date, int time)
     LocalToUTC(date, time, &utcd, &utct);
 
     /* Convert from Remind representation to year/mon/day */
-    FromJulian(utcd, &y, &m, &d);
+    FromDSE(utcd, &y, &m, &d);
 
-    /* Convert to a true Julian date -- sorry for the name clashes! */
-    jd = jtime(y, m, d, (utct / 60), (utct % 60), 0);   
+    /* Convert to a Julian date */
+    jd = jtime(y, m, d, (utct / 60), (utct % 60), 0);
 
     /* Calculate moon phase */
     mp = 360.0 * phase(jd, NULL, NULL, NULL, NULL, NULL, NULL);
@@ -553,8 +538,8 @@ void HuntPhase(int startdate, int starttim, int phas, int *date, int *time)
     LocalToUTC(startdate, starttim, &utcd, &utct);
 
     /* Convert from Remind representation to year/mon/day */
-    FromJulian(utcd, &y, &m, &d);
-    /* Convert to a true Julian date -- sorry for the name clashes! */
+    FromDSE(utcd, &y, &m, &d);
+    /* Convert to a true Julian date */
     jdorig = jtime(y, m, d, (utct / 60), (utct % 60), 0);   
     jd = jdorig - 45.0;
     nt1 = meanphase(jd, 0.0, &k1);
@@ -572,7 +557,7 @@ void HuntPhase(int startdate, int starttim, int phas, int *date, int *time)
     jyear(jd, &y, &m, &d);
     jhms(jd, &h, &min, &s);
 
-    d1 = Julian(y, m, d);
+    d1 = DSE(y, m, d);
     t1 = h*60 + min;
     UTCToLocal(d1, t1, date, time);
 }
