@@ -6,7 +6,8 @@
 /*  the data structures for OMITted dates.                     */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2022 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2023 by Dianne Skoll                    */
+/*  SPDX-License-Identifier: GPL-2.0-only                      */
 /*                                                             */
 /***************************************************************/
 
@@ -28,6 +29,8 @@ static void InsertIntoSortedArray (int *array, int num, int key);
 static int FullOmitArray[MAX_FULL_OMITS];
 static int PartialOmitArray[MAX_PARTIAL_OMITS];
 
+/* WeekdayOmits is declared in global.h */
+
 /* How many of each omit types do we have? */
 static int NumFullOmits, NumPartialOmits;
 
@@ -37,6 +40,7 @@ typedef struct omitcontext {
     int numfull, numpart;
     int *fullsave;
     int *partsave;
+    int weekdaysave;
 } OmitContext;
 
 /* The stack of saved omit contexts */
@@ -52,6 +56,7 @@ static OmitContext *SavedOmitContexts = NULL;
 int ClearGlobalOmits(void)
 {
     NumFullOmits = NumPartialOmits = 0;
+    WeekdayOmits = 0;
     return OK;
 }
 
@@ -113,6 +118,7 @@ int PushOmitContext(ParsePtr p)
 
     context->numfull = NumFullOmits;
     context->numpart = NumPartialOmits;
+    context->weekdaysave = WeekdayOmits;
     context->fullsave = malloc(NumFullOmits * sizeof(int));
     if (NumFullOmits && !context->fullsave) {
 	free(context);
@@ -154,6 +160,7 @@ int PopOmitContext(ParsePtr p)
     if (!c) return E_POP_NO_PUSH;
     NumFullOmits = c->numfull;
     NumPartialOmits = c->numpart;
+    WeekdayOmits = c->weekdaysave;
 
 /* Copy the context over */
     for (i=0; i<NumFullOmits; i++)
@@ -181,7 +188,7 @@ int PopOmitContext(ParsePtr p)
 /*  OK or an error code.                                       */
 /*                                                             */
 /***************************************************************/
-int IsOmitted(int jul, int localomit, char const *omitfunc, int *omit)
+int IsOmitted(int dse, int localomit, char const *omitfunc, int *omit)
 {
     int y, m, d;
 
@@ -193,7 +200,7 @@ int IsOmitted(int jul, int localomit, char const *omitfunc, int *omit)
 	int r;
 	Value v;
 
-	FromJulian(jul, &y, &m, &d);
+	FromDSE(dse, &y, &m, &d);
 	sprintf(expr, "%s('%04d-%02d-%02d')",
 		omitfunc, y, m+1, d);
 	s = expr;
@@ -208,18 +215,24 @@ int IsOmitted(int jul, int localomit, char const *omitfunc, int *omit)
     }
 
     /* Is it omitted because of local omits? */
-    if (localomit & (1 << (jul % 7))) {
+    if (localomit & (1 << (dse % 7))) {
 	*omit = 1;
 	return OK;
     }
 
+    /* Is it omitted because of global weekday omits? */
+    if (WeekdayOmits & (1 << (dse % 7))) {
+        *omit = 1;
+        return OK;
+    }
+
     /* Is it omitted because of fully-specified omits? */
-    if (BexistsIntArray(FullOmitArray, NumFullOmits, jul)) {
+    if (BexistsIntArray(FullOmitArray, NumFullOmits, dse)) {
 	*omit = 1;
 	return OK;
     }
 
-    FromJulian(jul, &y, &m, &d);
+    FromDSE(dse, &y, &m, &d);
     if (BexistsIntArray(PartialOmitArray, NumPartialOmits, (m << 5) + d)) {
 	*omit = 1;
 	return OK;
@@ -289,6 +302,7 @@ int DoOmit(ParsePtr p)
     int syndrome;
     int not_first_token = -1;
     int start, end, tmp;
+    int wd = 0;
 
     int mc, dc;
 
@@ -301,9 +315,15 @@ int DoOmit(ParsePtr p)
 	if ( (r=ParseToken(p, &buf)) ) return r;
 	FindToken(DBufValue(&buf), &tok);
 	switch (tok.type) {
-	case T_Dumpvars:
-	    if (not_first_token) return E_PARSE_ERR;
+        case T_WkDay:
 	    DBufFree(&buf);
+            if (wd & (1 << tok.val)) return E_WD_TWICE;
+            wd |= (1 << tok.val);
+            break;
+
+	case T_Dumpvars:
+	    DBufFree(&buf);
+	    if (not_first_token) return E_PARSE_ERR;
 	    r = VerifyEoln(p);
 	    if (r != OK) return r;
 	    DumpOmits();
@@ -314,7 +334,7 @@ int DoOmit(ParsePtr p)
 	    if (y[seen_through] != NO_YR) return E_YR_TWICE;
 	    if (m[seen_through] != NO_MON) return E_MON_TWICE;
 	    if (d[seen_through] != NO_DAY) return E_DAY_TWICE;
-	    FromJulian(tok.val, &y[seen_through], &m[seen_through], &d[seen_through]);
+	    FromDSE(tok.val, &y[seen_through], &m[seen_through], &d[seen_through]);
 	    break;
 
 	case T_Year:
@@ -341,6 +361,7 @@ int DoOmit(ParsePtr p)
 
 	case T_Through:
 	    DBufFree(&buf);
+            if (wd) return E_PARSE_ERR;
             if (seen_through) return E_UNTIL_TWICE;
             seen_through = 1;
             break;
@@ -363,6 +384,18 @@ int DoOmit(ParsePtr p)
 	}
     }
 
+    if (wd) {
+        if (y[0] != NO_YR || m[0] != NO_MON || d[0] != NO_DAY) {
+            return E_PARSE_ERR;
+        }
+        if ((WeekdayOmits | wd) == 0x7F) {
+            return E_2MANY_LOCALOMIT;
+        }
+        WeekdayOmits |= wd;
+        if (tok.type == T_Tag || tok.type == T_Duration || tok.type == T_RemType || tok.type == T_Priority) return E_PARSE_AS_REM;
+        return OK;
+    }
+
     if (!seen_through) {
         /* We must have at least a month */
         if (m[0] == NO_MON) return E_SPEC_MON;
@@ -425,8 +458,8 @@ int DoOmit(ParsePtr p)
         /* Full OMITs */
 	if (d[0] > DaysInMonth(m[0], y[0])) return E_BAD_DATE;
 	if (d[1] > DaysInMonth(m[1], y[1])) return E_BAD_DATE;
-        start = Julian(y[0], m[0], d[0]);
-        end   = Julian(y[1], m[1], d[1]);
+        start = DSE(y[0], m[0], d[0]);
+        end   = DSE(y[1], m[1], d[1]);
         if (end < start) {
             Eprint("Error: THROUGH date earlier than start date");
             return E_BAD_DATE;
@@ -447,11 +480,11 @@ int DoOmit(ParsePtr p)
 }
 
 int
-AddGlobalOmit(int jul)
+AddGlobalOmit(int dse)
 {
     if (NumFullOmits == MAX_FULL_OMITS) return E_2MANY_FULL;
-    if (!BexistsIntArray(FullOmitArray, NumFullOmits, jul)) {
-        InsertIntoSortedArray(FullOmitArray, NumFullOmits, jul);
+    if (!BexistsIntArray(FullOmitArray, NumFullOmits, dse)) {
+        InsertIntoSortedArray(FullOmitArray, NumFullOmits, dse);
         NumFullOmits++;
     }
     return OK;
@@ -467,7 +500,7 @@ DumpOmits(void)
 	printf("\tNone.\n");
     } else {
 	for (i=0; i<NumFullOmits; i++) {
-	    FromJulian(FullOmitArray[i], &y, &m, &d);
+	    FromDSE(FullOmitArray[i], &y, &m, &d);
 	    printf("\t%04d%c%02d%c%02d\n",
 		    y, DateSep, m+1, DateSep, d);
 	}
@@ -482,5 +515,15 @@ DumpOmits(void)
 	    printf("\t%02d%c%02d\n", m+1, DateSep, d);
 	}
     }
+    printf("Global Weekday OMITs:\n");
+    if (WeekdayOmits == 0) {
+        printf("\tNone.\n");
+    } else {
+        for (i=0; i<7; i++) {
+            if (WeekdayOmits & (1<<i)) {
+                printf("\t%s\n", EnglishDayName[i]);
+            }
+        }
+    }
 }
 
