@@ -5,7 +5,8 @@
 /*  Function Prototypes.                                       */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2022 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2023 by Dianne Skoll                    */
+/*  SPDX-License-Identifier: GPL-2.0-only                      */
 /*                                                             */
 /***************************************************************/
 
@@ -26,6 +27,7 @@
 
 int CallUserFunc (char const *name, int nargs, ParsePtr p);
 int DoFset (ParsePtr p);
+int DoFunset (ParsePtr p);
 void ProduceCalendar (void);
 char const *SimpleTime (int tim);
 char const *CalendarTime (int tim, int duration);
@@ -33,11 +35,11 @@ int DoRem (ParsePtr p);
 int DoFlush (ParsePtr p);
 void DoExit (ParsePtr p);
 int ParseRem (ParsePtr s, Trigger *trig, TimeTrig *tim, int save_in_globals);
-int TriggerReminder (ParsePtr p, Trigger *t, TimeTrig *tim, int jul);
-int ShouldTriggerReminder (Trigger *t, TimeTrig *tim, int jul, int *err);
-int DoSubst (ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul, int mode);
-int DoSubstFromString (char const *source, DynamicBuffer *dbuf, int jul, int tim);
-int ParseLiteralDate (char const **s, int *jul, int *tim);
+int TriggerReminder (ParsePtr p, Trigger *t, TimeTrig *tim, int dse);
+int ShouldTriggerReminder (Trigger *t, TimeTrig *tim, int dse, int *err);
+int DoSubst (ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int dse, int mode);
+int DoSubstFromString (char const *source, DynamicBuffer *dbuf, int dse, int tim);
+int ParseLiteralDate (char const **s, int *dse, int *tim);
 int ParseLiteralTime (char const **s, int *tim);
 int EvalExpr (char const **e, Value *v, ParsePtr p);
 int DoCoerce (char type, Value *v);
@@ -49,13 +51,14 @@ int DoInclude (ParsePtr p, enum TokTypes tok);
 int DoIncludeCmd (ParsePtr p);
 int IncludeFile (char const *fname);
 int GetAccessDate (char const *file);
-int SetAccessDate (char const *fname, int jul);
+int SetAccessDate (char const *fname, int dse);
 int TopLevel (void);
 int CallFunc (BuiltinFunc *f, int nargs);
 void InitRemind (int argc, char const *argv[]);
 void Usage (void);
-int Julian (int year, int month, int day);
-void FromJulian (int jul, int *y, int *m, int *d);
+int DSE (int year, int month, int day);
+void FromDSE (int dse, int *y, int *m, int *d);
+int JulianToGregorianOffset(int y, int m);
 int ParseChar (ParsePtr p, int *err, int peek);
 int ParseToken (ParsePtr p, DynamicBuffer *dbuf);
 int ParseIdentifier (ParsePtr p, DynamicBuffer *dbuf);
@@ -85,7 +88,7 @@ int DoClear (ParsePtr p);
 int DestroyOmitContexts (void);
 int PushOmitContext (ParsePtr p);
 int PopOmitContext (ParsePtr p);
-int IsOmitted (int jul, int localomit, char const *omitfunc, int *omit);
+int IsOmitted (int dse, int localomit, char const *omitfunc, int *omit);
 int DoOmit (ParsePtr p);
 int QueueReminder (ParsePtr p, Trigger *trig, TimeTrig *tim, char const *sched);
 void HandleQueuedReminders (void);
@@ -95,9 +98,7 @@ void FindNumericToken (char const *s, Token *t);
 int ComputeTrigger (int today, Trigger *trig, TimeTrig *tim, int *err, int save_in_globals);
 int ComputeTriggerNoAdjustDuration (int today, Trigger *trig, TimeTrig *tim, int *err, int save_in_globals, int duration_days);
 int AdjustTriggerForDuration(int today, int r, Trigger *trig, TimeTrig *tim, int save_in_globals);
-int ComputeScanStart(int today, Trigger *trig, TimeTrig *tt);
 char *StrnCpy (char *dest, char const *source, int n);
-int StrMatch (char const *s1, char const *s2, int n);
 int StrinCmp (char const *s1, char const *s2, int n);
 char *StrDup (char const *s);
 int StrCmpi (char const *s1, char const *s2);
@@ -119,24 +120,24 @@ unsigned int HashVal (char const *str);
 int DateOK (int y, int m, int d);
 Operator *FindOperator (char const *name, Operator where[], int num);
 BuiltinFunc *FindFunc (char const *name, BuiltinFunc where[], int num);
-int InsertIntoSortBuffer (int jul, int tim, char const *body, int typ, int prio);
+int InsertIntoSortBuffer (int dse, int tim, char const *body, int typ, int prio);
 void IssueSortedReminders (void);
 int UserFuncExists (char const *fn);
-void JulToHeb (int jul, int *hy, int *hm, int *hd);
+void DSEToHeb (int dse, int *hy, int *hm, int *hd);
 int HebNameToNum (char const *mname);
 char const *HebMonthName (int m, int y);
 int RoshHashana (int i);
 long DaysToHebYear (int y);
 int DaysInHebYear (int y);
 char const *DaysInHebMonths (int ylen);
-int HebToJul (int hy, int hm, int hd);
+int HebToDSE (int hy, int hm, int hd);
 int GetValidHebDate (int yin, int min, int din, int adarbehave, int *mout, int *dout, int yahr);
-int GetNextHebrewDate (int julstart, int hm, int hd, int yahr, int adarbehave, int *ans);
+int GetNextHebrewDate (int dsestart, int hm, int hd, int yahr, int adarbehave, int *ans);
 int ComputeJahr (int y, int m, int d, int *ans);
 int GetSysVar (char const *name, Value *val);
 int SetSysVar (char const *name, Value *val);
 void DumpSysVarByName (char const *name);
-int CalcMinsFromUTC (int jul, int tim, int *mins, int *isdst);
+int CalcMinsFromUTC (int dse, int tim, int *mins, int *isdst);
 void FillParagraph (char const *s);
 void LocalToUTC (int locdate, int loctime, int *utcdate, int *utctime);
 void UTCToLocal (int utcdate, int utctime, int *locdate, int *loctime);
@@ -155,17 +156,17 @@ void SaveLastTimeTrig(TimeTrig const *t);
 void SaveAllTriggerInfo(Trigger const *t, TimeTrig const *tt, int trigdate, int trigtime, int valid);
 
 void PerIterationInit(void);
-char const *Decolorize(int r, int g, int b);
-char const *Colorize(int r, int g, int b);
+char const *Decolorize(void);
+char const *Colorize(int r, int g, int b, int bg, int clamp);
 void PrintJSONString(char const *s);
 void PrintJSONKeyPairInt(char const *name, int val);
 void PrintJSONKeyPairString(char const *name, char const *val);
-void PrintJSONKeyPairDate(char const *name, int jul);
+void PrintJSONKeyPairDate(char const *name, int dse);
 void PrintJSONKeyPairDateTime(char const *name, int dt);
 void PrintJSONKeyPairTime(char const *name, int t);
 void System(char const *cmd);
 int ShellEscape(char const *in, DynamicBuffer *out);
-int AddGlobalOmit(int jul);
+int AddGlobalOmit(int dse);
 void set_lat_and_long_from_components(void);
 void set_components_from_lat_and_long(void);
 
@@ -174,6 +175,11 @@ char const *get_month_name(int mon);
 
 int push_call(char const *filename, char const *func, int lineno);
 void clear_callstack(void);
-int have_callstack(void);
 int print_callstack(FILE *fp);
 void pop_call(void);
+#ifdef REM_USE_WCHAR
+#define _XOPEN_SOURCE 600
+#include <wctype.h>
+#include <wchar.h>
+void PutWideChar(wchar_t const wc);
+#endif
