@@ -6,7 +6,8 @@
 /*  classifying the tokens parsed.                             */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2022 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2023 by Dianne Skoll                    */
+/*  SPDX-License-Identifier: GPL-2.0-only                      */
 /*                                                             */
 /***************************************************************/
 
@@ -62,6 +63,7 @@ Token TokArray[] = {
     { "friday",		3,	T_WkDay,	4 },
     { "from",		4,	T_Scanfrom,	FROM_TYPE },
     { "fset",		4,	T_Fset,		0 },
+    { "funset",         6,      T_Funset,       0 },
     { "if",		2,	T_If,		0 },
     { "iftrig",		6,	T_IfTrig,	0 },
     { "in",             2,      T_In,           0 },
@@ -79,6 +81,7 @@ Token TokArray[] = {
     { "monday",		3,	T_WkDay,	0 },
     { "msf",		3,	T_RemType,	MSF_TYPE },
     { "msg",		3,	T_RemType,	MSG_TYPE },
+    { "noqueue",        7,      T_NoQueue,      0 },
     { "november",	3,	T_Month,	10 },
     { "october",	3,	T_Month,	9 },
     { "omit",		3,	T_Omit,		0 },
@@ -238,16 +241,16 @@ void FindNumericToken(char const *s, Token *t)
 	/* If we hit a '-' or a '/', we may have a date or a datetime */
 	if (*s == '-' || *s == '/') {
 	    char const *p = s_orig;
-	    int jul, tim;
-	    if (ParseLiteralDate(&p, &jul, &tim) == OK) {
+	    int dse, tim;
+	    if (ParseLiteralDate(&p, &dse, &tim) == OK) {
 		if (*p) return;
 		if (tim == NO_TIME) {
 		    t->type = T_Date;
-		    t->val = jul;
+		    t->val = dse;
 		    return;
 		}
 		t->type = T_DateTime;
-		t->val = MINUTES_PER_DAY * jul + tim;
+		t->val = MINUTES_PER_DAY * dse + tim;
 	    }
 	    return;
 	}
@@ -255,7 +258,6 @@ void FindNumericToken(char const *s, Token *t)
 	/* If we hit a comma, swallow it.  This allows stuff
 	   like Jan 6, 1998 */
 	if (*s == ',') {
-	    s++;
 	    /* Classify the number we've got */
 	    if (t->val >= BASE && t->val <= BASE+YR_RANGE) t->type = T_Year;
 	    else if (t->val >= 1 && t->val <= 31) t->type = T_Day;
@@ -354,7 +356,8 @@ static int TokStrCmp(Token const *t, char const *s)
     register int r;
     char const *tk = t->name;
     while(*tk && *s && !(*s == ',' && *(s+1) == 0)) {
-	r = tolower(*tk) - tolower(*s);
+        /* t->name is already lower-case */
+	r = *tk - tolower(*s);
 	tk++;
 	s++;
 	if (r) return r;
@@ -362,5 +365,5 @@ static int TokStrCmp(Token const *t, char const *s)
     /* Ignore trailing commas on s */
 
     if (!*s || (*s == ',' && !*(s+1))) return 0;
-    return (tolower(*tk) - tolower(*s));
+    return (*tk - tolower(*s));
 }
