@@ -5,7 +5,8 @@
 /*  Type definitions all dumped here.                          */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2022 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2023 by Dianne Skoll                    */
+/*  SPDX-License-Identifier: GPL-2.0-only                      */
 /*                                                             */
 /***************************************************************/
 
@@ -77,6 +78,7 @@ typedef struct {
     int eventduration;           /* Original event duration (minutes) */
     int maybe_uncomputable;      /* Suppress "can't compute trigger" warnings */
     int addomit;                 /* Add trigger date to global OMITs */
+    int noqueue;                 /* Don't queue even if timed */
     char sched[VAR_NAME_LEN+1];  /* Scheduling function */
     char warn[VAR_NAME_LEN+1];   /* Warning function    */
     char omitfunc[VAR_NAME_LEN+1]; /* OMITFUNC function */
@@ -155,8 +157,8 @@ enum TokTypes
   /* Commands first */
   T_Rem, T_Push, T_Pop, T_Preserve, T_Include, T_IncludeR, T_IncludeCmd, T_If, T_Else, T_EndIf,
   T_IfTrig, T_ErrMsg,
-  T_Set, T_UnSet, T_Fset, T_Omit, T_Banner, T_Exit,
-  T_AddOmit,
+  T_Set, T_UnSet, T_Fset, T_Funset, T_Omit, T_Banner, T_Exit,
+  T_AddOmit, T_NoQueue,
   T_WkDay,
   T_Month, T_Time, T_Date, T_DateTime,
   T_Skip, T_At, T_RemType, T_Until, T_Year, T_Day, T_Rep, T_Delta,
@@ -238,6 +240,6 @@ typedef struct {
 /* Pure JSON */
 #define PSCAL_LEVEL3  3
 
-#define TERMINAL_BACKGROUND_UNKNOWN 0
-#define TERMINAL_BACKGROUND_DARK    1
-#define TERMINAL_BACKGROUND_LIGHT   2
+#define TERMINAL_BACKGROUND_UNKNOWN -1
+#define TERMINAL_BACKGROUND_DARK    0
+#define TERMINAL_BACKGROUND_LIGHT   1
