@@ -6,7 +6,8 @@
 /*  functions.                                                 */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2022 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2023 by Dianne Skoll                    */
+/*  SPDX-License-Identifier: GPL-2.0-only                      */
 /*                                                             */
 /***************************************************************/
 
@@ -53,6 +54,34 @@ static void FSet (UserFunc *f);
 static int SetUpLocalVars (UserFunc *f);
 static void DestroyLocalVals (UserFunc *f);
 
+/***************************************************************/
+/*                                                             */
+/*  DoFunset                                                   */
+/*                                                             */
+/*  Undefine a user-defined function - the FUNSET command.     */
+/*                                                             */
+/***************************************************************/
+int DoFunset(ParsePtr p)
+{
+    int r;
+    int seen_one = 0;
+    DynamicBuffer buf;
+    DBufInit(&buf);
+
+    while(1) {
+        r = ParseIdentifier(p, &buf);
+        if (r == E_EOLN) {
+            DBufFree(&buf);
+            break;
+        }
+        seen_one = 1;
+        FUnset(DBufValue(&buf));
+        DBufFree(&buf);
+    }
+    if (seen_one) return OK;
+    return E_PARSE_ERR;
+}
+
 /***************************************************************/
 /*                                                             */
 /*  DoFset                                                     */
@@ -97,6 +126,7 @@ int DoFset(ParsePtr p)
     }
     func->filename = StrDup(FileName);
     if (!func->filename) {
+        free(func);
         return E_NO_MEM;
     }
     func->lineno = LineNo;
@@ -153,7 +183,7 @@ int DoFset(ParsePtr p)
     /* Allow an optional = sign: FSET f(x) = x*x */
     c = ParseNonSpaceChar(p, &r, 1);
     if (c == '=') {
-	c = ParseNonSpaceChar(p, &r, 0);
+	(void) ParseNonSpaceChar(p, &r, 0);
     }
     /* Copy the text over */
     if (p->isnested) {
