@@ -5,7 +5,8 @@
 /*  Useful utility functions.                                  */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2022 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2023 by Dianne Skoll                    */
+/*  SPDX-License-Identifier: GPL-2.0-only                      */
 /*                                                             */
 /***************************************************************/
 
@@ -24,8 +25,6 @@ static char const DontEscapeMe[] =
 #include "globals.h"
 #include "protos.h"
 
-#define UPPER(c) toupper(c)
-
 /***************************************************************/
 /*                                                             */
 /*  StrnCpy                                                    */
@@ -42,22 +41,6 @@ char *StrnCpy(char *dest, char const *source, int n)
     return odest;
 }
 
-/***************************************************************/
-/*                                                             */
-/*  StrMatch                                                   */
-/*                                                             */
-/*  Checks that two strings match (case-insensitive) to at     */
-/*  least the specified number of characters, or the length    */
-/*  of the first string, whichever is greater.                 */
-/*                                                             */
-/***************************************************************/
-int StrMatch(char const *s1, char const *s2, int n)
-{
-    int l;
-    if ((l = strlen(s1)) < n) return 0;
-    return !StrinCmp(s1, s2, l);
-}
-
 /***************************************************************/
 /*                                                             */
 /*  StrinCmp - compare strings, case-insensitive               */
@@ -68,12 +51,12 @@ int StrinCmp(char const *s1, char const *s2, int n)
     register int r;
     while (n && *s1 && *s2) {
 	n--;
-	r = UPPER(*s1) - UPPER(*s2);
+	r = toupper(*s1) - toupper(*s2);
 	if (r) return r;
 	s1++;
 	s2++;
     }
-    if (n) return (UPPER(*s1) - UPPER(*s2)); else return 0;
+    if (n) return (toupper(*s1) - toupper(*s2)); else return 0;
 }
 
 /***************************************************************/
@@ -102,12 +85,12 @@ int StrCmpi(char const *s1, char const *s2)
 {
     int r;
     while (*s1 && *s2) {
-	r = UPPER(*s1) - UPPER(*s2);
+	r = toupper(*s1) - toupper(*s2);
 	if (r) return r;
 	s1++;
 	s2++;
     }
-    return UPPER(*s1) - UPPER(*s2);
+    return toupper(*s1) - toupper(*s2);
 }
 
 /***************************************************************/
@@ -224,13 +207,6 @@ clear_callstack(void)
     callstack = NULL;
 }
 
-int
-have_callstack(void)
-{
-    if (callstack) return 1;
-    return 0;
-}
-
 static void
 print_callstack_aux(FILE *fp, cs *entry)
 {
